############################################################################
    ##
    ## Copyright (C) 2021 The Qt Company Ltd.
    ## Contact: https://www.qt.io/licensing/
    ##
    ## This file is part of the Qt for Python examples of the Qt Toolkit.
    ##
    ## $QT_BEGIN_LICENSE:BSD$
    ## Commercial License Usage
    ## Licensees holding valid commercial Qt licenses may use this file in
    ## accordance with the commercial license agreement provided with the
    ## Software or, alternatively, in accordance with the terms contained in
    ## a written agreement between you and The Qt Company. For licensing terms
    ## and conditions see https://www.qt.io/terms-conditions. For further
    ## information use the contact form at https://www.qt.io/contact-us.
    ##
    ## BSD License Usage
    ## Alternatively, you may use this file under the terms of the BSD license
    ## as follows:
    ##
    ## "Redistribution and use in source and binary forms, with or without
    ## modification, are permitted provided that the following conditions are
    ## met:
    ##   * Redistributions of source code must retain the above copyright
    ##     notice, this list of conditions and the following disclaimer.
    ##   * Redistributions in binary form must reproduce the above copyright
    ##     notice, this list of conditions and the following disclaimer in
    ##     the documentation and/or other materials provided with the
    ##     distribution.
    ##   * Neither the name of The Qt Company Ltd nor the names of its
    ##     contributors may be used to endorse or promote products derived
    ##     from this software without specific prior written permission.
    ##
    ##
    ## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    ## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    ## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
    ## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
    ## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
    ## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    ## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    ## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    ## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    ## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
    ##
    ## $QT_END_LICENSE$
    ##
############################################################################
# We use the os module to get this file's directory
import os
CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))

# We import our custom widget
from template import Template

# We import any necessary PySide6 modules
from PySide6.QtCore import QObject, Slot
from PySide6.QtGui import QAction, QIcon
from PySide6.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QLineEdit
from PySide6.QtDesigner import (QExtensionManager,
    QExtensionFactory,
    QDesignerCustomWidgetInterface,
    QPyDesignerTaskMenuExtension,
    QDesignerFormWindowInterface)

# Extension Type | Id
E_ID = {
    'TaskMenu': 'org.qt-project.Qt.Designer.TaskMenu'
}

#============================================================================#
# ExtensionFactory                                                           #
#----------------------------------------------------------------------------#
class Factory(QExtensionFactory):
    """The Extension Factory is used to register extensions"""
    def __init__(self, manager, _type):
        super().__init__(manager)
        self.type = _type

    def createExtension(self, obj, iid, parent):
        if isinstance(obj, Template) and iid == E_ID[self.type]:
            if self.type == 'TaskMenu':return TaskMenu(obj, parent)
        return None


#============================================================================#
# TaskMenuExtension                                                          #
#----------------------------------------------------------------------------#
class Dialog(QDialog):
    """This is the pop up dialog"""
    def __init__(self, parent):
        super().__init__(parent)
        self._widget = parent

        layout = QVBoxLayout(self)

        self._edited = QLineEdit(self)
        layout.addWidget(self._edited)

        button_box = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
        button_box.accepted.connect(self.accept)
        button_box.rejected.connect(self.reject)
        layout.addWidget(button_box)

class TaskMenu(QPyDesignerTaskMenuExtension):
    def __init__(self, widget, parent):
        """
            This function gets called when you first
            request a menu in QtDesigner by right
            clicking on your custom widget.
        """
        super().__init__(parent)
        self._widget = widget

        # Each action is a menu entry
        # You still have to add them bellow to
        # the 'taskActions' method and connect them to a method
        self._dialogAction = QAction('Edit Something...', None)
        self._dialogAction.triggered.connect(self.popupDialog)

    def taskActions(self):
        """Adds actions to task menu"""
        return [self._dialogAction]

    def preferredEditAction(self):
        """Triggers when plugin is double clicked in QtDesigner"""
        return self._dialogAction

    @Slot()
    def popupDialog(self):
        """
            Pops up dialog

            This is just an example of how you can add a
            popup dialog to let the user further edit
            your plugin parameters.
        """
        dialog = Dialog(self._widget)
        if dialog.exec() == QDialog.Accepted:
            # here you could do something to your plugin
            # For example if your plugin had a property
            # called 'state', you could set it by doing:

            # self._widget.state = dialog._edited.text()
            # form = QDesignerFormWindowInterface.findFormWindow(self._widget)
            # if form:
                ## We force setting the property to see the changes
                ## reflected in the property editor in real time
                # form.cursor().setWidgetProperty(self._widget, "state", dialog._edited.text())

            pass

#============================================================================#
# CustomWidgetPlugin                                                         #
#----------------------------------------------------------------------------#
DOM_XML = """
<ui language='c++'>
    <widget class='Template' name='template'/>
</ui>
"""

class TemplatePlugin(QDesignerCustomWidgetInterface):
    def __init__(self):
        super().__init__()
        self._form_editor = None

    def createWidget(self, *args, **kwargs):
        t = Template(*args, **kwargs)
        return t

    def domXml(self):
        return DOM_XML

    def group(self):
        return 'Custom Widgets'

    def icon(self):
        imgLocation = os.path.join(CURRENT_DIR,'IconName')
        return QIcon(imgLocation)

    def includeFile(self):
        return 'template'

    def initialize(self, form_editor):
        if self.isInitialized():return
        manager = form_editor.extensionManager()
        if not manager: return
        self._form_editor = form_editor
        manager.registerExtensions(Factory(manager,'TaskMenu'), E_ID['TaskMenu'])

    def isContainer(self):
        return False

    def isInitialized(self):
        return self._form_editor is not None

    def name(self):
        return 'Template'

    def toolTip(self):
        return 'tooltip'

    def whatsThis(self):
        return self.toolTip()



from PySide6.QtWidgets import *
from PySide6.QtCore import *
from PySide6.QtGui import *
import os

def getMainWindow():
    # for widget in QApplication.allWidgets():
    #     print(widget)
    for widget in QApplication.topLevelWidgets():
        if isinstance(widget, QMainWindow):
            return widget
    return None

class JumpButton(QPushButton):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.marginsAnim = QPropertyAnimation(self, b'geometry')
        self.marginsAnim.setDuration(300)
        self.marginsAnim.setEasingCurve(QEasingCurve.OutElastic)

    def enterEvent(self,event):
        self.marginsAnim.setDirection(self.marginsAnim.Forward)
        if self.marginsAnim.state() == self.marginsAnim.State.Stopped:
            rect = self.geometry()
            self.marginsAnim.setStartValue(rect)
            rect+=QMargins(0,10,0,0)
            self.marginsAnim.setEndValue(rect)
            if self.isEnabled():self.marginsAnim.start()
        QPushButton.enterEvent(self, event)

    def leaveEvent(self,event):
        self.marginsAnim.setDirection(self.marginsAnim.Backward)
        if self.marginsAnim.state() == self.marginsAnim.State.Stopped and self.isEnabled(): self.marginsAnim.start()
        QPushButton.leaveEvent(self, event)

class ToolButton(QToolButton):
    def __init__(self, *args, **kwargs):
        QToolButton.__init__(self, *args, **kwargs)
        self.tip = QLabel(getMainWindow())
        self.tip.setStyleSheet(" ".join(F"""
            color: rgb(219, 203, 189);
            background-color: rgb(135, 67, 29);
            padding: 10px;
            border: 2px solid #c87941;
            border-radius: 5px;
        """.split()))


        self.opacityFx = QGraphicsOpacityEffect(self.tip)
        self.opacityFx.setOpacity(0)
        self.tip.setGraphicsEffect(self.opacityFx)
        self.opacityAnim = QPropertyAnimation(self.opacityFx, b'opacity')
        self.opacityAnim.setDuration(250)
        self.opacityAnim.setStartValue(0)
        self.opacityAnim.setEndValue(1)

        self.posAnim = QPropertyAnimation(self.tip, b'pos')
        self.posAnim.setDuration(400)
        self.posAnim.setEasingCurve(QEasingCurve.OutElastic)

        self.animGroup = QParallelAnimationGroup(self.tip)
        self.animGroup.addAnimation(self.opacityAnim)
        self.animGroup.addAnimation(self.posAnim)

    def event(self,event):
        t=event.type()
        if t == QEvent.ToolTip:
            return True
        elif t == QEvent.Show or t == QEvent.Leave:
            self.tip.hide()
        elif t == QEvent.Enter:
            self.tip.setText(self.toolTip())
            self.tip.adjustSize()
            self.tip.show()
            gw=self.geometry().width()
            gh=self.geometry().height()
            x=self.pos().x()#+gw+15
            y=self.pos().y()#+gh*1.9

            if self.group().objectName() == 'mainPageGroup':
                x=(x+gw*.8)-self.tip.geometry().width()*.5
                y+=gh+7
                self.posAnim.setStartValue(QPoint(x,y*2))
            else:
                x+=gw+15
                y+=gh*1.9
                self.posAnim.setStartValue(QPoint(x*2,y))
            self.posAnim.setEndValue(QPoint(x,y))

            if self.isEnabled():self.animGroup.start()
        return QToolButton.event(self, event)

class DragDrop(QAbstractItemView):
    dropped = Signal(list)
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setAcceptDrops(True)

    def mousePressEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            mData = QMimeData()
            urls=[]
            for item in self.selectedItems():
                if hasattr(item,'file'):
                    urls.append(QUrl.fromLocalFile(item.file))
            if len(urls):
                mData.setUrls(urls)
                drag = QDrag(self)
                drag.setMimeData(mData)
                drag.setHotSpot(event.position().toPoint() - self.rect().topLeft())
                Qt.DropAction = drag.exec(Qt.CopyAction)
        return super().mousePressEvent(event)

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dragMoveEvent(self, event):
        if event.mimeData().hasUrls():
            event.setDropAction(Qt.CopyAction)
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        if event.mimeData().hasUrls() and event.source() is None:
            event.setDropAction(Qt.CopyAction)
            event.accept()
            links = []
            for url in event.mimeData().urls():
                links.append(str(url.toLocalFile()))
            self.dropped.emit(links)
        else:
            event.ignore()

class ListView(QListWidget,DragDrop):
    def __init__(self, parent=None):
        super().__init__(parent)

    def getItems(self):
        return (self.item(i) for i in range(self.count()))

    def removeItem(self,file):
        for i, item in enumerate(self.getItems()):
            if item.file == file:
                self.takeItem(i)
                break

    def dragEnterEvent(self, event):DragDrop.dragEnterEvent(self, event)
    def dragMoveEvent(self, event):DragDrop.dragMoveEvent(self, event)
    def dropEvent(self, event):DragDrop.dropEvent(self, event)

class PathsList(ListView):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.mainWindow = getMainWindow()
        self.menu = QMenu('Paths Menu',self)
        self.menu.setCursor(Qt.PointingHandCursor)
        self.menu.addAction('Add',self.mainWindow.addPluginPath)
        self.menu.addAction('Delete',self.mainWindow.removePluginPath)

    def removeCurrentItem(self):
        for i, item in enumerate(self.getItems()):
            if item == self.currentItem():
                self.takeItem(i)
                return item

    def contextMenuEvent(self,event):
        underMouse = self.itemAt(event.pos())
        if underMouse is None:# Empty area
            [action.setVisible(action.text() == 'Add') for action in self.menu.actions()]
        else:# Item
            [action.setVisible(action.text() == 'Delete') for action in self.menu.actions()]
        self.menu.popup(event.globalPos())
        event.accept()

class TreeView(QTreeWidget,DragDrop):
    droppedQrc = Signal(list)
    droppedImg = Signal(dict)
    droppedModule = Signal(list)
    def __init__(self, parent=None):
        super().__init__(parent)
        self.imgTypes = ['.png' ,'.ico' ,'.jpg' ,'.gif' ,'.svg']
        self.qrcParent = None
        self.prefixParent = None
        self.mainWindow = getMainWindow()

        # icons
        fileIcon = QIcon(":/icons/img/cil-file.png")
        folderIcon = QIcon(":/icons/img/cil-folder-open.png")
        pencilIcon = QIcon(":/icons/img/cil-pencil.png")
        refreshIcon = QIcon(":/icons/img/cil-loop-circular.png")
        imageIcon = QIcon(":/icons/img/cil-satelite.png")
        plusIcon = QIcon(":/icons/img/cil-plus.png")

        # menus
        self.popup = None
        self.tinted = False
        self.menu = QMenu('Menu',self)
        self.menu.setCursor(Qt.PointingHandCursor)

        self.menu.addAction(pencilIcon,'Open',self.open)
        self.menu.addAction(fileIcon,'New Module',self.mainWindow.newModule)
        self.menu.addAction(folderIcon,'Add Module',self.mainWindow.addModuleDialogue)
        self.menu.addAction(plusIcon,'Add Class',self.mainWindow.addClass)
        self.menu.addAction(folderIcon,'Add Images',self.mainWindow.addImagesDialogue)
        self.menu.addAction(folderIcon,'Add Prefix',self.mainWindow.addPrefix)
        self.menu.addAction(folderIcon,'Add Qrc',self.mainWindow.addQrc)
        self.menu.addAction(folderIcon,'Create Qrc',self.mainWindow.createQrc)
        self.menu.addAction(pencilIcon,'Tint',self.tint)
        self.menu.addAction(pencilIcon,'Multiply',self.multiply)
        self.menu.addAction(refreshIcon,'Restore',self.restore)
        self.menu.addAction(imageIcon,'Make ICO',self.makeIco)
        self.menu.addAction(imageIcon,'Set as project icon',self.setIcon)
        self.menu.addAction(fileIcon,'Rename',self.rename)
        self.menu.addAction(fileIcon,'Delete',self.delete)

    def mousePressEvent(self, event):
        QTreeWidget.mousePressEvent(self, event)
        si = self.selectedImages()
        sm = self.selectedModules()
        if si:selectedItems = si
        elif sm:selectedItems = sm
        else:return True
        if event.buttons() == Qt.LeftButton:
            mData = QMimeData()
            urls=[]
            for item in selectedItems:
                if hasattr(item,'file'):
                    urls.append(QUrl.fromLocalFile(item.file))
            if len(urls):
                mData.setUrls(urls)
                drag = QDrag(self)
                drag.setMimeData(mData)
                drag.setHotSpot(event.position().toPoint() - self.rect().topLeft())
                Qt.DropAction = drag.exec(Qt.CopyAction)

    def removeTopLevelItem(self,item,col=0):
        self.takeTopLevelItem(self.indexOfTopLevelItem(item))

    def makeIco(self):self.mainWindow.makeIco(self.selectedImages())
    def tint(self):self.mainWindow.tintImagesDlg(self.selectedImages(),0)
    def multiply(self):self.mainWindow.tintImagesDlg(self.selectedImages(),1)
    def restore(self):self.mainWindow.restoreImages(self.selectedImages())
    def setIcon(self):self.mainWindow.setIcon(self.currentItem())

    def open(self):
        if self.popup == 'module':self.mainWindow.openModule(self.currentItem())
        if self.popup == 'class':self.mainWindow.openClass(self.currentItem())

    def delete(self):
        if self.popup == 'qrc': self.mainWindow.deleteQrcs(self.selectedQrcs())
        if self.popup == 'prefix': self.mainWindow.deletePrefixes(self.selectedPrefixes())
        if self.popup == 'image': self.mainWindow.deleteImages(self.selectedImages())
        if self.popup == 'module': self.mainWindow.deleteModules()

    def rename(self):
        if self.popup == 'qrc': self.mainWindow.renameQrc(self.currentItem())# rename prefix
        if self.popup == 'prefix': self.mainWindow.addPrefix(add=False)# rename prefix
        if self.popup == 'image': self.mainWindow.renameImage(self.currentItem())# rename prefix
        if self.popup == 'module': self.mainWindow.renameModule(self.currentItem())# rename prefix

    def enableActions(self,actions=None,state=True):
        for action in self.menu.actions():
            if actions:
                if action.text() not in actions:
                    continue
            action.setEnabled(state)

    def visibleActions(self,actions=None,state=True):
        for action in self.menu.actions():
            if actions:
                if action.text() not in actions:
                    continue
            action.setVisible(state)

    def selectedQrcs(self):
        """returns selected qrc files"""
        if self.objectName() == 'qrcTree':
            return [i for i in self.selectedItems() if i.type()==0]

    def selectedPrefixes(self):
        """returns selected prefixes"""
        if self.objectName() == 'qrcTree':
            return [i for i in self.selectedItems() if i.type()==1]

    def selectedImages(self):
        """returns selected images"""
        if self.objectName() == 'qrcTree':
            return [si for si in self.selectedItems() if si.type()==2]

    def selectedModules(self):
        """returns selected modules"""
        if self.objectName() == 'moduleTree':
            return [i for i in self.selectedItems() if i.type()==0]

    def selectedClasses(self):
        """returns selected classes"""
        if self.objectName() == 'moduleTree':
            return [i for i in self.selectedItems() if i.type()==1]

    def contextMenuEvent(self,event):
        underMouse = self.itemAt(event.pos())
        self.visibleActions(state=0)
        if self.objectName() == 'qrcTree':
            if underMouse:
                if underMouse.type()==2:# image
                    self.popup = 'image'
                    self.visibleActions([
                        'Restore',
                        'Rename',
                        'Delete',
                    ])
                    # if any image is tinted enable undo tint
                    self.enableActions(['Restore'],0)
                    for img in self.selectedImages():
                        if '.ico' in img.file:
                            self.visibleActions(['Set as project icon'])
                        else:
                            self.visibleActions([
                                'Tint',
                                'Multiply',
                                'Make ICO'
                            ])
                        if img.tinted:
                            self.enableActions(['Restore'])
                elif underMouse.type()==1:# prefix
                    self.popup = 'prefix'
                    self.visibleActions([
                        'Add Images',
                        'Rename',
                        'Delete'
                    ])
                elif underMouse.type()==0:# qrc
                    self.popup = 'qrc'
                    self.visibleActions([
                        'Add Prefix',
                        'Rename',
                        'Delete'
                    ])
            else:# empty
                self.popup = None
                self.visibleActions(['Add Qrc','Create Qrc'])
        else:# moduleTree
            if underMouse:
                if underMouse.type()==1:# class
                    self.popup = 'class'
                    if self.mainWindow.hasSublime():
                        self.visibleActions(['Open'])
                    else:
                        event.ignore()
                        return
                elif underMouse.type()==0:# module
                    self.popup = 'module'
                    self.visibleActions([
                        'Add Class',
                        'Rename',
                        'Delete'
                    ])
                    if self.mainWindow.hasSublime():
                        self.visibleActions(['Open'])
            else:# empty
                self.popup = None
                self.visibleActions([
                    'New Module',
                    'Add Module'
                ])
        self.menu.popup(event.globalPos())
        event.accept()

    def getUrls(self,event):
        return [str(url.toLocalFile()) for url in event.mimeData().urls()]

    def urlType(self,event):
        if not event.mimeData().hasUrls():return None
        extList = []
        for url in self.getUrls(event):
            ext = os.path.splitext(url)[1]
            if ext not in extList and ext not in self.imgTypes and len(extList): return 'multiple'
            extList.append(ext)
        if all([e=='.py' for e in extList]):return 'module'
        if all([e=='.qrc' for e in extList]):return 'qrc'
        if all([e in self.imgTypes for e in extList]):return 'img'
        # if all([not len(e) for e in extList]):return 'folder'
        print(extList)
        return None

    def dragEnterEvent(self, event):DragDrop.dragEnterEvent(self, event)

    def dragMoveEvent(self, event):
        urlType = self.urlType(event)
        if not urlType or event.source():
            event.ignore()
            return
        elif (urlType=='img' or urlType=='qrc') and self.objectName() != 'qrcTree':
            event.ignore()
            return
        elif urlType=='module' and self.objectName() != 'moduleTree':
            event.ignore()
            return
        else:
            event.setDropAction(Qt.CopyAction)
            event.accept()

        hovered = self.itemAt(event.position().toPoint())
        # De-Select items
        for item in self.selectedItems():item.setSelected(0)
        shouldSelect = []

        if hovered:
            if self.objectName() == 'qrcTree':
                self.mainWindow.ui.statusbar.showMessage('')
                self.prefixParent=None
                if hovered.type()==2:# hovered image
                    self.prefixParent = hovered.parent()
                    self.qrcParent = hovered.parent().parent()
                elif hovered.type()==1:# hovered prefix
                    self.prefixParent=hovered
                    self.qrcParent=hovered.parent()
                else:# hovered qrc
                    self.qrcParent=hovered

                shouldSelect.append(self.qrcParent)
                if self.prefixParent:
                    shouldSelect.append(self.prefixParent)
                else:
                    self.mainWindow.ui.statusbar.showMessage('A new prefix will be created!')
        else:# hovering empty area
            if self.topLevelItem(0):# not empty
                if self.objectName() == 'moduleTree':
                    shouldSelect.append(self.topLevelItem(0))
                elif self.objectName() == 'qrcTree':
                    if urlType == 'img':
                        qrc_file = self.topLevelItem(0)
                        shouldSelect.append(qrc_file)
                        self.qrcParent=qrc_file

                        first_prefix = qrc_file.child(0)
                        if first_prefix:
                            shouldSelect.append(first_prefix)
                            self.prefixParent=first_prefix
                        else:
                            self.prefixParent=None
                            self.mainWindow.ui.statusbar.showMessage('A new prefix will be created!')

        [si.setSelected(1) for si in shouldSelect]
        # DragDrop.dragMoveEvent(self, event)

    def dropEvent(self, event):
        urlType = self.urlType(event)
        if urlType and event.source() is None:
            event.setDropAction(Qt.CopyAction)
            event.accept()
            if urlType=='img':
                self.droppedImg.emit({
                'qrc':self.qrcParent,
                'prefix':self.prefixParent,# if None we need to create a 'newPrefix'
                'imgs':self.getUrls(event)
                })
            if urlType=='qrc':self.droppedQrc.emit(self.getUrls(event))
            if urlType=='module':self.droppedModule.emit(self.getUrls(event))
        else:
            event.ignore()







# -*- coding: utf-8 -*-



################################################################################

## Form generated from reading UI file 'gui.ui'

##

## Created by: Qt User Interface Compiler version 6.2.2

##

## WARNING! All changes made in this file will be lost when recompiling UI file!

################################################################################



from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,

    QMetaObject, QObject, QPoint, QRect,

    QSize, QTime, QUrl, Qt)

from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,

    QCursor, QFont, QFontDatabase, QGradient,

    QIcon, QImage, QKeySequence, QLinearGradient,

    QPainter, QPalette, QPixmap, QRadialGradient,

    QTransform)

from PySide6.QtWidgets import (QApplication, QCheckBox, QFormLayout, QFrame,

    QHBoxLayout, QLabel, QLineEdit, QListWidgetItem,

    QMainWindow, QPushButton, QScrollArea, QSizePolicy,

    QSpacerItem, QSpinBox, QStatusBar, QToolButton,

    QVBoxLayout, QWidget)



from myWidgets import (JumpButton, PathsList)

import resources_rc



class Ui_MainWindow(object):

    def setupUi(self, MainWindow):

        if not MainWindow.objectName():

            MainWindow.setObjectName(u"MainWindow")

        MainWindow.resize(593, 662)

        MainWindow.setAcceptDrops(True)

        icon = QIcon()

        icon.addFile(u":/images/img/mpi_logo_icon.ico", QSize(), QIcon.Normal, QIcon.Off)

        MainWindow.setWindowIcon(icon)

        MainWindow.setStyleSheet(u"QWidget{\n"

"	font: 600 14pt \"Dosis SemiBold\";\n"

"	color: rgb(219, 203, 189);\n"

"	background:transparent;\n"

"outline: none; \n"

"}\n"

"QAbstractScrollArea{\n"

"	background-color: transparent;\n"

"}\n"

"*[isTitle=\"true\"]{\n"

"	border-bottom:2px solid;\n"

"	color: rgb(41, 0, 1);\n"

"	font: 800 24pt \"Dosis ExtraBold\";\n"

"	padding-bottom:5px;\n"

"}\n"

"QMainWindow,QWidget#centralwidget,QStatusBar{\n"

"	background-color: qlineargradient(spread:pad, x1:0.42, y1:0, x2:0.528, y2:1, stop:0 rgba(135, 67, 29, 255), stop:1 rgba(41, 0, 1, 255));\n"

"	border-radius:10px;\n"

"}\n"

"QStatusBar{\n"

"	background-color: rgb(41, 0, 1);\n"

"}\n"

"QStatusBar::item {\n"

"    border: none;\n"

"}\n"

"QStatusBar QLabel#extraMsg {\n"

"	background-color: rgb(135, 67, 29);\n"

"}\n"

"QSizeGrip {\n"

"	image: url(:/icons/img/cil-size-grip.png);\n"

"     width: 16px;\n"

"     height: 16px;\n"

"	margin:0px 10px 10px 0px;\n"

"}\n"

"\n"

"QMenu {	\n"

"    border: 2px solid rgb(200, 121, 65);\n"

"	border-radius:3px;\n"

"	padding:5px;\n"

""

                        "}\n"

"\n"

"QMenu::item {\n"

"    /* sets background of menu item. set this to something non-transparent\n"

"        if you want menu color and menu item color to be different */\n"

"    background-color: transparent;\n"

"}\n"

"QMenu::item:selected {\n"

"	background-color: rgb(135, 67, 29);	\n"

"	color: rgb(219, 203, 189);\n"

"}\n"

"QMenu::item:disabled {\n"

"	background-color: rgb(219, 203, 189);\n"

"   	color: rgb(135, 67, 29);\n"

"}\n"

"\n"

"QPushButton{\n"

"	background-color: qlineargradient(spread:pad, x1:0.489, y1:1, x2:0.476864, y2:0, stop:0 rgba(41, 0, 1, 255), stop:0.670455 rgba(135, 67, 29, 255));\n"

"padding:4px;\n"

"border-radius:4px;\n"

"}\n"

"QAbstractButton:hover,QAbstractButton:pressed, QAbstractButton:checked {\n"

"	background-color: qlineargradient(spread:pad, x1:0.46, y1:1, x2:0.454136, y2:0.017, stop:0.420455 rgba(135, 67, 29, 255), stop:0.920455 rgba(200, 121, 65, 255));\n"

"}\n"

"\n"

"QPushButton:pressed, QPushButton:checked{\n"

"border:2px solid rgb(200, 121, 65);\n"

"}\n"

"\n"

"QAbstract"

                        "Button:disabled{\n"

"	background-color: rgb(135, 67, 29);\n"

"	color: rgb(41, 0, 1);\n"

"}\n"

"QToolButton:disabled{\n"

"	background-color: rgb(135, 67, 29);\n"

"}\n"

"\n"

"QToolButton{\n"

"background:transparent;\n"

"border:none;\n"

"}\n"

"\n"

"QToolButton:checked,QToolButton:pressed{\n"

"border-bottom:2px solid rgb(200, 121, 65);\n"

"}\n"

"QHeaderView::section {	\n"

"	color: rgb(200, 121, 65);\n"

"    border:none;\n"

" background:transparent;\n"

"}\n"

"\n"

"\n"

"QAbstractItemView::item{\n"

"padding:8px;\n"

"	color: rgb(219, 203, 189);\n"

"border-radius:8px;\n"

"margin-right:5px;\n"

"}\n"

"\n"

"QAbstractItemView::item:hover{\n"

"	border:1px solid rgb(219, 203, 189);\n"

"}\n"

"\n"

"QAbstractItemView::item:selected, QAbstractItemView::item:selected:active,QAbstractItemView::item:selected:!active{\n"

"	background-color: qlineargradient(spread:repeat, x1:0, y1:1, x2:0, y2:0, stop:0.0959821 rgba(135, 67, 29, 255), stop:0.151786 rgba(200, 121, 65, 255), stop:0.928571 rgba(174, 99, 51, 255), stop:1 rgba(135, 67, 29,"

                        " 255));\n"

"	color: rgb(41, 0, 1);\n"

"}\n"

"QListView::item{\n"

"border-bottom:1px solid #000;	\n"

"}\n"

"\n"

"QListView#designerPathsList::item{\n"

"padding:0px;\n"

"border:none;\n"

"margin:0px;\n"

"font-size:9pt;\n"

"}\n"

"\n"

"QComboBox, QLineEdit {\n"

"border:1px solid rgb(200, 121, 65);\n"

"}\n"

"QTextEdit,QLineEdit:focus, QComboBox::on, QComboBox QListView::item{\n"

"	background-color: rgb(41, 0, 1);\n"

"}\n"

"\n"

"QLineEdit:disabled{\n"

"	background-color: rgb(219, 203, 189);\n"

"	color: rgb(41, 0, 1);\n"

"}\n"

"\n"

"QToolTip { 		\n"

"	font: 600 14pt \"Dosis SemiBold\";\n"

"    color: rgb(219, 203, 189);\n"

"    background-color: rgb(135, 67, 29);\n"

"    padding: 10px;\n"

"    border: 2px solid #c87941;\n"

"    border-radius: 5px;\n"

"}\n"

"\n"

"\n"

"\n"

"QAbstractSlider {	\n"

"	background-color: rgb(41, 0, 1);\n"

"}\n"

"QAbstractSlider::handle{\n"

"	background-color: rgb(135, 67, 29);\n"

"}\n"

"\n"

"QAbstractSlider:horizontal {\n"

"    border: none;\n"

"    height: 12px;\n"

"    margin: 0px 21px 0 21"

                        "px;\n"

"	border-radius: 0px;\n"

"}\n"

"QAbstractSlider::handle:horizontal {\n"

"    min-width: 25px;\n"

"}\n"

"QAbstractSlider::add-line:horizontal {\n"

"    border: none;\n"

"	border-left:2px solid rgba(255,255,255,120);\n"

"    background: rgba(0,0,0,80);\n"

"    width: 20px;\n"

"	border-top-right-radius: 4px;\n"

"    border-bottom-right-radius: 4px;\n"

"    subcontrol-position: right;\n"

"    subcontrol-origin: margin;\n"

"}\n"

"QAbstractSlider::sub-line:horizontal {\n"

"    border: none;\n"

"	border-right:2px solid rgba(255,255,255,120);\n"

"    background: rgba(0,0,0,80);\n"

"    width: 20px;\n"

"	border-top-left-radius: 4px;\n"

"    border-bottom-left-radius: 4px;\n"

"    subcontrol-position: left;\n"

"    subcontrol-origin: margin;\n"

"}\n"

"QAbstractSlider::up-arrow:horizontal, QAbstractSlider::down-arrow:horizontal\n"

"{\n"

"     background: none;\n"

"}\n"

"QAbstractSlider::add-page:horizontal, QAbstractSlider::sub-page:horizontal\n"

"{\n"

"     background: none;\n"

"}\n"

" QAbstractSlider:vertical {\n"

""

                        "	border: none;	\n"

"    width: 12px;\n"

"    margin: 21px 0 21px 0;\n"

"	border-radius: 0px;\n"

" }\n"

" QAbstractSlider::handle:vertical {	\n"

"    min-height: 25px;\n"

" }\n"

" QAbstractSlider::add-line:vertical {\n"

"     border: none;\n"

"	border-top:2px solid rgba(255,255,255,120);\n"

"    background: rgba(0,0,0,80);\n"

"     height: 20px;\n"

"	border-bottom-left-radius: 4px;\n"

"    border-bottom-right-radius: 4px;\n"

"     subcontrol-position: bottom;\n"

"     subcontrol-origin: margin;\n"

" }\n"

" QAbstractSlider::sub-line:vertical {\n"

"	border: none;\n"

"	border-bottom:2px solid rgba(255,255,255,120);\n"

"    background:  rgba(0,0,0,80);\n"

"     height: 20px;\n"

"	border-top-left-radius: 4px;\n"

"    border-top-right-radius: 4px;\n"

"     subcontrol-position: top;\n"

"     subcontrol-origin: margin;\n"

" }\n"

" QAbstractSlider::up-arrow:vertical, QAbstractSlider::down-arrow:vertical {\n"

"     background: none;\n"

" }\n"

"\n"

" QAbstractSlider::add-page:vertical, QAbstractSlider::sub-page:vertical {"

                        "\n"

"     background: none;\n"

" }\n"

"QSlider::handle::horizontal{\n"

"width:25px;\n"

"}\n"

"QSlider::groove{\n"

"	background-color: rgb(41, 0, 1);\n"

"	border:1px solid rgb(135, 67, 29);\n"

"}\n"

"\n"

"")

        self.actionPotatoes = QAction(MainWindow)

        self.actionPotatoes.setObjectName(u"actionPotatoes")

        self.centralwidget = QWidget(MainWindow)

        self.centralwidget.setObjectName(u"centralwidget")

        self.verticalLayout = QVBoxLayout(self.centralwidget)

        self.verticalLayout.setObjectName(u"verticalLayout")

        self.verticalLayout.setContentsMargins(-1, -1, -1, 0)

        self.header = QWidget(self.centralwidget)

        self.header.setObjectName(u"header")

        self.header.setCursor(QCursor(Qt.SizeAllCursor))

        self.horizontalLayout_3 = QHBoxLayout(self.header)

        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")

        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)

        self.TitleLyt = QVBoxLayout()

        self.TitleLyt.setSpacing(0)

        self.TitleLyt.setObjectName(u"TitleLyt")

        self.TitleLyt.setContentsMargins(-1, -1, -1, 0)

        self.mainTitle = QLabel(self.header)

        self.mainTitle.setObjectName(u"mainTitle")

        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Minimum)

        sizePolicy.setHorizontalStretch(0)

        sizePolicy.setVerticalStretch(0)

        sizePolicy.setHeightForWidth(self.mainTitle.sizePolicy().hasHeightForWidth())

        self.mainTitle.setSizePolicy(sizePolicy)

        self.mainTitle.setStyleSheet(u"font-size:22pt;")



        self.TitleLyt.addWidget(self.mainTitle)



        self.secTitle = QLabel(self.header)

        self.secTitle.setObjectName(u"secTitle")

        sizePolicy.setHeightForWidth(self.secTitle.sizePolicy().hasHeightForWidth())

        self.secTitle.setSizePolicy(sizePolicy)

        self.secTitle.setStyleSheet(u"font-size:12pt;")



        self.TitleLyt.addWidget(self.secTitle)





        self.horizontalLayout_3.addLayout(self.TitleLyt)



        self.horizontalSpacer_7 = QSpacerItem(337, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)



        self.horizontalLayout_3.addItem(self.horizontalSpacer_7)



        self.logo = QLabel(self.header)

        self.logo.setObjectName(u"logo")

        sizePolicy1 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)

        sizePolicy1.setHorizontalStretch(0)

        sizePolicy1.setVerticalStretch(0)

        sizePolicy1.setHeightForWidth(self.logo.sizePolicy().hasHeightForWidth())

        self.logo.setSizePolicy(sizePolicy1)

        self.logo.setMaximumSize(QSize(1000, 1000))

        self.logo.setCursor(QCursor(Qt.PointingHandCursor))

        self.logo.setPixmap(QPixmap(u":/images/img/mpi_logo_small.png"))



        self.horizontalLayout_3.addWidget(self.logo)



        self.verticalLayout_2 = QVBoxLayout()

        self.verticalLayout_2.setSpacing(1)

        self.verticalLayout_2.setObjectName(u"verticalLayout_2")

        self.verticalLayout_2.setContentsMargins(-1, 4, -1, -1)

        self.closeBtn = QToolButton(self.header)

        self.closeBtn.setObjectName(u"closeBtn")

        sizePolicy1.setHeightForWidth(self.closeBtn.sizePolicy().hasHeightForWidth())

        self.closeBtn.setSizePolicy(sizePolicy1)

        self.closeBtn.setMaximumSize(QSize(16777215, 24))

        self.closeBtn.setCursor(QCursor(Qt.PointingHandCursor))

        self.closeBtn.setIconSize(QSize(8, 8))



        self.verticalLayout_2.addWidget(self.closeBtn)



        self.verticalSpacer_6 = QSpacerItem(20, 3, QSizePolicy.Minimum, QSizePolicy.Minimum)



        self.verticalLayout_2.addItem(self.verticalSpacer_6)



        self.minimizeBtn = QToolButton(self.header)

        self.minimizeBtn.setObjectName(u"minimizeBtn")

        sizePolicy1.setHeightForWidth(self.minimizeBtn.sizePolicy().hasHeightForWidth())

        self.minimizeBtn.setSizePolicy(sizePolicy1)

        self.minimizeBtn.setMaximumSize(QSize(16777215, 24))

        self.minimizeBtn.setCursor(QCursor(Qt.PointingHandCursor))

        self.minimizeBtn.setIconSize(QSize(8, 8))



        self.verticalLayout_2.addWidget(self.minimizeBtn)





        self.horizontalLayout_3.addLayout(self.verticalLayout_2)





        self.verticalLayout.addWidget(self.header)



        self.scrollArea = QScrollArea(self.centralwidget)

        self.scrollArea.setObjectName(u"scrollArea")

        self.scrollArea.setFrameShape(QFrame.NoFrame)

        self.scrollArea.setWidgetResizable(True)

        self.scrollAreaWidgetContents = QWidget()

        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")

        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 575, 517))

        self.formLayout = QFormLayout(self.scrollAreaWidgetContents)

        self.formLayout.setObjectName(u"formLayout")

        self.label = QLabel(self.scrollAreaWidgetContents)

        self.label.setObjectName(u"label")



        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.label)



        self.pluginName = QLineEdit(self.scrollAreaWidgetContents)

        self.pluginName.setObjectName(u"pluginName")



        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.pluginName)



        self.label_2 = QLabel(self.scrollAreaWidgetContents)

        self.label_2.setObjectName(u"label_2")



        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.label_2)



        self.QtClass = QLineEdit(self.scrollAreaWidgetContents)

        self.QtClass.setObjectName(u"QtClass")



        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.QtClass)



        self.label_9 = QLabel(self.scrollAreaWidgetContents)

        self.label_9.setObjectName(u"label_9")



        self.formLayout.setWidget(2, QFormLayout.LabelRole, self.label_9)



        self.horizontalLayout = QHBoxLayout()

        self.horizontalLayout.setObjectName(u"horizontalLayout")

        self.label_3 = QLabel(self.scrollAreaWidgetContents)

        self.label_3.setObjectName(u"label_3")



        self.horizontalLayout.addWidget(self.label_3)



        self.geoW = QSpinBox(self.scrollAreaWidgetContents)

        self.geoW.setObjectName(u"geoW")

        self.geoW.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.geoW.setMaximum(8000)

        self.geoW.setValue(100)



        self.horizontalLayout.addWidget(self.geoW)



        self.label_10 = QLabel(self.scrollAreaWidgetContents)

        self.label_10.setObjectName(u"label_10")



        self.horizontalLayout.addWidget(self.label_10)



        self.geoH = QSpinBox(self.scrollAreaWidgetContents)

        self.geoH.setObjectName(u"geoH")

        self.geoH.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.geoH.setMaximum(8000)

        self.geoH.setValue(100)



        self.horizontalLayout.addWidget(self.geoH)



        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)



        self.horizontalLayout.addItem(self.horizontalSpacer)





        self.formLayout.setLayout(2, QFormLayout.FieldRole, self.horizontalLayout)



        self.label_8 = QLabel(self.scrollAreaWidgetContents)

        self.label_8.setObjectName(u"label_8")



        self.formLayout.setWidget(3, QFormLayout.LabelRole, self.label_8)



        self.propList = PathsList(self.scrollAreaWidgetContents)

        self.propList.setObjectName(u"propList")

        self.propList.setStyleSheet(u"border:1px solid rgb(200, 121, 65);\n"

"")

        self.propList.setFrameShape(QFrame.NoFrame)



        self.formLayout.setWidget(3, QFormLayout.FieldRole, self.propList)



        self.label_4 = QLabel(self.scrollAreaWidgetContents)

        self.label_4.setObjectName(u"label_4")



        self.formLayout.setWidget(4, QFormLayout.LabelRole, self.label_4)



        self.tooltipIn = QLineEdit(self.scrollAreaWidgetContents)

        self.tooltipIn.setObjectName(u"tooltipIn")



        self.formLayout.setWidget(4, QFormLayout.FieldRole, self.tooltipIn)



        self.label_5 = QLabel(self.scrollAreaWidgetContents)

        self.label_5.setObjectName(u"label_5")



        self.formLayout.setWidget(5, QFormLayout.LabelRole, self.label_5)



        self.horizontalLayout_20 = QHBoxLayout()

        self.horizontalLayout_20.setSpacing(8)

        self.horizontalLayout_20.setObjectName(u"horizontalLayout_20")

        self.newPIcon = QLabel(self.scrollAreaWidgetContents)

        self.newPIcon.setObjectName(u"newPIcon")

        self.newPIcon.setPixmap(QPixmap(u":/images/img/mpi_logo_icon.ico"))

        self.newPIcon.setScaledContents(False)



        self.horizontalLayout_20.addWidget(self.newPIcon)



        self.newPIconBrowse = QPushButton(self.scrollAreaWidgetContents)

        self.newPIconBrowse.setObjectName(u"newPIconBrowse")

        self.newPIconBrowse.setMinimumSize(QSize(24, 0))

        self.newPIconBrowse.setCursor(QCursor(Qt.PointingHandCursor))



        self.horizontalLayout_20.addWidget(self.newPIconBrowse)



        self.horizontalSpacer_6 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)



        self.horizontalLayout_20.addItem(self.horizontalSpacer_6)





        self.formLayout.setLayout(5, QFormLayout.FieldRole, self.horizontalLayout_20)



        self.label_7 = QLabel(self.scrollAreaWidgetContents)

        self.label_7.setObjectName(u"label_7")



        self.formLayout.setWidget(6, QFormLayout.LabelRole, self.label_7)



        self.widgetGroup = QLineEdit(self.scrollAreaWidgetContents)

        self.widgetGroup.setObjectName(u"widgetGroup")



        self.formLayout.setWidget(6, QFormLayout.FieldRole, self.widgetGroup)



        self.isContainerCb = QCheckBox(self.scrollAreaWidgetContents)

        self.isContainerCb.setObjectName(u"isContainerCb")

        self.isContainerCb.setIconSize(QSize(32, 32))



        self.formLayout.setWidget(8, QFormLayout.FieldRole, self.isContainerCb)



        self.label_6 = QLabel(self.scrollAreaWidgetContents)

        self.label_6.setObjectName(u"label_6")



        self.formLayout.setWidget(10, QFormLayout.LabelRole, self.label_6)



        self.horizontalLayout_21 = QHBoxLayout()

        self.horizontalLayout_21.setSpacing(8)

        self.horizontalLayout_21.setObjectName(u"horizontalLayout_21")

        self.regPath = QLineEdit(self.scrollAreaWidgetContents)

        self.regPath.setObjectName(u"regPath")



        self.horizontalLayout_21.addWidget(self.regPath)



        self.regOpen = QPushButton(self.scrollAreaWidgetContents)

        self.regOpen.setObjectName(u"regOpen")

        self.regOpen.setMinimumSize(QSize(24, 0))

        self.regOpen.setCursor(QCursor(Qt.PointingHandCursor))



        self.horizontalLayout_21.addWidget(self.regOpen)



        self.addToExistingCb = QCheckBox(self.scrollAreaWidgetContents)

        self.addToExistingCb.setObjectName(u"addToExistingCb")

        self.addToExistingCb.setIconSize(QSize(32, 32))



        self.horizontalLayout_21.addWidget(self.addToExistingCb)





        self.formLayout.setLayout(10, QFormLayout.FieldRole, self.horizontalLayout_21)



        self.addMenuCb = QCheckBox(self.scrollAreaWidgetContents)

        self.addMenuCb.setObjectName(u"addMenuCb")



        self.formLayout.setWidget(9, QFormLayout.FieldRole, self.addMenuCb)



        self.isMultiCb = QCheckBox(self.scrollAreaWidgetContents)

        self.isMultiCb.setObjectName(u"isMultiCb")



        self.formLayout.setWidget(7, QFormLayout.FieldRole, self.isMultiCb)



        self.scrollArea.setWidget(self.scrollAreaWidgetContents)



        self.verticalLayout.addWidget(self.scrollArea)



        self.createPluginBtn = JumpButton(self.centralwidget)

        self.createPluginBtn.setObjectName(u"createPluginBtn")

        self.createPluginBtn.setCursor(QCursor(Qt.PointingHandCursor))

        icon1 = QIcon()

        icon1.addFile(u":/icons/img/cil-file.png", QSize(), QIcon.Normal, QIcon.Off)

        self.createPluginBtn.setIcon(icon1)

        self.createPluginBtn.setIconSize(QSize(16, 16))



        self.verticalLayout.addWidget(self.createPluginBtn)



        MainWindow.setCentralWidget(self.centralwidget)

        self.statusbar = QStatusBar(MainWindow)

        self.statusbar.setObjectName(u"statusbar")

        self.statusbar.setStyleSheet(u"font-size:12pt;")

        self.statusbar.setSizeGripEnabled(True)

        MainWindow.setStatusBar(self.statusbar)



        self.retranslateUi(MainWindow)

        self.closeBtn.clicked.connect(MainWindow.close)

        self.minimizeBtn.clicked.connect(MainWindow.showMinimized)



        QMetaObject.connectSlotsByName(MainWindow)

    # setupUi



    def retranslateUi(self, MainWindow):

        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MadQt Plugin Creator", None))

        self.actionPotatoes.setText(QCoreApplication.translate("MainWindow", u"Potatoes", None))

#if QT_CONFIG(statustip)

        self.header.setStatusTip(QCoreApplication.translate("MainWindow", u"Double click for full screen", None))

#endif // QT_CONFIG(statustip)

#if QT_CONFIG(statustip)

        self.mainTitle.setStatusTip(QCoreApplication.translate("MainWindow", u"Start page", None))

#endif // QT_CONFIG(statustip)

        self.mainTitle.setText(QCoreApplication.translate("MainWindow", u"MadQt", None))

        self.secTitle.setText(QCoreApplication.translate("MainWindow", u"    Plugin Creator", None))

#if QT_CONFIG(statustip)

        self.logo.setStatusTip(QCoreApplication.translate("MainWindow", u"Documentation", None))

#endif // QT_CONFIG(statustip)

        self.logo.setText("")

#if QT_CONFIG(tooltip)

        self.closeBtn.setToolTip(QCoreApplication.translate("MainWindow", u"Close", None))

#endif // QT_CONFIG(tooltip)

#if QT_CONFIG(statustip)

        self.closeBtn.setStatusTip(QCoreApplication.translate("MainWindow", u"Close", None))

#endif // QT_CONFIG(statustip)

        self.closeBtn.setText(QCoreApplication.translate("MainWindow", u"x", None))

#if QT_CONFIG(tooltip)

        self.minimizeBtn.setToolTip(QCoreApplication.translate("MainWindow", u"Minimize", None))

#endif // QT_CONFIG(tooltip)

#if QT_CONFIG(statustip)

        self.minimizeBtn.setStatusTip(QCoreApplication.translate("MainWindow", u"Minimize", None))

#endif // QT_CONFIG(statustip)

        self.minimizeBtn.setText(QCoreApplication.translate("MainWindow", u"-", None))

        self.label.setText(QCoreApplication.translate("MainWindow", u"Class Name", None))

#if QT_CONFIG(statustip)

        self.pluginName.setStatusTip(QCoreApplication.translate("MainWindow", u"Your plugin name", None))

#endif // QT_CONFIG(statustip)

        self.pluginName.setPlaceholderText(QCoreApplication.translate("MainWindow", u"MyAwesomePlugin", None))

        self.label_2.setText(QCoreApplication.translate("MainWindow", u"Qt Class", None))

#if QT_CONFIG(statustip)

        self.QtClass.setStatusTip(QCoreApplication.translate("MainWindow", u"Qt Class your plugin inherits from", None))

#endif // QT_CONFIG(statustip)

        self.QtClass.setPlaceholderText(QCoreApplication.translate("MainWindow", u"QWidget", None))

#if QT_CONFIG(statustip)

        self.label_9.setStatusTip(QCoreApplication.translate("MainWindow", u"Paths to QtDesigner Plugins", None))

#endif // QT_CONFIG(statustip)

        self.label_9.setText(QCoreApplication.translate("MainWindow", u"Size Hint", None))

        self.label_3.setText(QCoreApplication.translate("MainWindow", u"Width", None))

#if QT_CONFIG(statustip)

        self.geoW.setStatusTip(QCoreApplication.translate("MainWindow", u"Width", None))

#endif // QT_CONFIG(statustip)

        self.label_10.setText(QCoreApplication.translate("MainWindow", u"Height", None))

#if QT_CONFIG(statustip)

        self.geoH.setStatusTip(QCoreApplication.translate("MainWindow", u"Height", None))

#endif // QT_CONFIG(statustip)

#if QT_CONFIG(statustip)

        self.label_8.setStatusTip(QCoreApplication.translate("MainWindow", u"Paths to QtDesigner Plugins", None))

#endif // QT_CONFIG(statustip)

        self.label_8.setText(QCoreApplication.translate("MainWindow", u"Properties", None))

#if QT_CONFIG(statustip)

        self.propList.setStatusTip(QCoreApplication.translate("MainWindow", u"Right click to add/remove properties", None))

#endif // QT_CONFIG(statustip)

        self.label_4.setText(QCoreApplication.translate("MainWindow", u"Tooltip", None))

#if QT_CONFIG(statustip)

        self.tooltipIn.setStatusTip(QCoreApplication.translate("MainWindow", u"QtDesigner tooltip", None))

#endif // QT_CONFIG(statustip)

        self.tooltipIn.setText("")

        self.tooltipIn.setPlaceholderText(QCoreApplication.translate("MainWindow", u"My plugin is...", None))

        self.label_5.setText(QCoreApplication.translate("MainWindow", u"Icon", None))

        self.newPIcon.setText("")

#if QT_CONFIG(statustip)

        self.newPIconBrowse.setStatusTip(QCoreApplication.translate("MainWindow", u"Select image file -> will be converted to .ico", None))

#endif // QT_CONFIG(statustip)

        self.newPIconBrowse.setText(QCoreApplication.translate("MainWindow", u"...", None))

        self.label_7.setText(QCoreApplication.translate("MainWindow", u"Group", None))

#if QT_CONFIG(statustip)

        self.widgetGroup.setStatusTip(QCoreApplication.translate("MainWindow", u"Group name in QtDesigner Widget box", None))

#endif // QT_CONFIG(statustip)

        self.widgetGroup.setText("")

        self.widgetGroup.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Custom Widgets", None))

#if QT_CONFIG(statustip)

        self.isContainerCb.setStatusTip(QCoreApplication.translate("MainWindow", u"Will it contain other widgets?", None))

#endif // QT_CONFIG(statustip)

        self.isContainerCb.setText(QCoreApplication.translate("MainWindow", u"Is Container", None))

        self.label_6.setText(QCoreApplication.translate("MainWindow", u"Registry", None))

#if QT_CONFIG(statustip)

        self.regPath.setStatusTip(QCoreApplication.translate("MainWindow", u"Files path", None))

#endif // QT_CONFIG(statustip)

        self.regPath.setText("")

        self.regPath.setPlaceholderText(QCoreApplication.translate("MainWindow", u"C:\\path\\to\\plugin\\registration", None))

#if QT_CONFIG(statustip)

        self.regOpen.setStatusTip(QCoreApplication.translate("MainWindow", u"Open path/file", None))

#endif // QT_CONFIG(statustip)

        self.regOpen.setText(QCoreApplication.translate("MainWindow", u"...", None))

#if QT_CONFIG(statustip)

        self.addToExistingCb.setStatusTip(QCoreApplication.translate("MainWindow", u"Add to an existing register ", None))

#endif // QT_CONFIG(statustip)

        self.addToExistingCb.setText(QCoreApplication.translate("MainWindow", u"Add to Existing", None))

        self.addMenuCb.setText(QCoreApplication.translate("MainWindow", u"Add Menu", None))

        self.isMultiCb.setText(QCoreApplication.translate("MainWindow", u"Is Multi Page", None))

#if QT_CONFIG(statustip)

        self.createPluginBtn.setStatusTip(QCoreApplication.translate("MainWindow", u"Create Plugin", None))

#endif // QT_CONFIG(statustip)

        self.createPluginBtn.setText(QCoreApplication.translate("MainWindow", u"Create Plugin", None))

#if QT_CONFIG(shortcut)

        self.createPluginBtn.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+N", None))

#endif // QT_CONFIG(shortcut)

    # retranslateUi






"""
Widget: HtButton
Version: 0.0.1

Contributors: Fabio Goncalves
Email: fabiogoncalves@live.co.uk

Description: A QPushButton with a hovered state and text
"""
import os
CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))

# Importing from the MadQt package Widgets folder
from MadQt.Widgets.hover_text_button.htbutton import HtButton

from PySide6.QtCore import QObject
from PySide6.QtGui import QIcon
from PySide6.QtDesigner import (QExtensionManager,
    QDesignerCustomWidgetInterface,
    QDesignerFormWindowInterface)

DOM_XML = """
<ui language='c++'>
    <widget class='HtButton' name='htbutton'/>
</ui>
"""

class HtButtonPlugin(QObject, QDesignerCustomWidgetInterface):
    def __init__(self):
        QObject.__init__(self)
        QDesignerCustomWidgetInterface.__init__(self)
        self._form_editor = None

    def createWidget(self, *args, **kwargs):
        widget = HtButton(*args, **kwargs)
        widget.defaultTextChanged.connect(self.defaultTextChanged)
        widget.hoverStateChanged.connect(self.hoverStateChanged)
        return widget

    def domXml(self):
        return DOM_XML

    def group(self):
        return 'Buttons'

    def icon(self):
        imgLocation = os.path.join(CURRENT_DIR,'htbutton.ico')
        return QIcon(imgLocation)

    def includeFile(self):
        # Importing from the MadQt package Widgets folder
        return 'MadQt.Widgets.hover_text_button.htbutton'

    def initialize(self, form_editor):
        self._form_editor = form_editor

    def isContainer(self):
        return False

    def isInitialized(self):
        return self._form_editor is not None

    def name(self):
        return 'HtButton'

    def toolTip(self):
        return 'A button that changes text when hovered'

    def whatsThis(self):
        return self.toolTip()

    def defaultTextChanged(self, text):
        widget = self.sender()
        if not widget or not isinstance(widget, HtButton):return
        if not widget.isWidgetType():return
        form = QDesignerFormWindowInterface.findFormWindow(widget)
        if form:
            form.cursor().setWidgetProperty(widget, "text", text)

    def hoverStateChanged(self, state):
        widget = self.sender()
        if not widget or not isinstance(widget, HtButton):return
        if not widget.isWidgetType():return
        form = QDesignerFormWindowInterface.findFormWindow(widget)
        if form:
            form.cursor().setWidgetProperty(widget, "hovered", state)



from PySide6.QtWidgets import *
from PySide6.QtCore import *
from PySide6.QtGui import *
import PySide6, os, json, shutil, webbrowser, tempfile, subprocess, sys, fileinput
import xml.etree.ElementTree as xml
import MadQt
from MadQt import Tools as Mt

# give this to users for them to create a shortcut
# https://pypi.org/project/psgshortcut/

# pyside6-uic gui.ui -o gui.py
# pyside6-rcc resources.qrc -o resources_rc.py

# To use only in development
# from pyside6_loader import PySide6Ui
# PySide6Ui('gui.ui').toPy()

import gui
# Globals
MadQt_Designer_Paths = []
window = None

def pySideDir():
    """returns the path to PySide6"""
    return os.path.dirname(PySide6.__file__)

def isTemp(path):
    temp_dir = os.path.join(tempfile.gettempdir(),'MadQt')
    return temp_dir in path

def tempDir():
    """
        returns the temp directory
        if it doesn't exist, it creates it
    """
    temp_dir = os.path.join(tempfile.gettempdir(),'MadQt')
    if not os.path.isdir(temp_dir): os.mkdir(temp_dir)
    return temp_dir

def customWidgetModule(_class,qt_class):
    return f"""from PySide6.QtWidgets import *
from PySide6.QtCore import *
from PySide6.QtGui import *

class {_class}({qt_class}):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)"""

def customWidgetClass(_class,qt_class):
    return f"""

class {_class}({qt_class}):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)"""


class ThreadF(QThread):
    # Basic Worker Thread for PyQt
    # Usage: ThreadF(parent,myFunctionName,*args,**kwargs)
    def __init__(self, parent=None, func=None, *args, **kwargs):
        QThread.__init__(self, parent)
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.start()
    def run(self):
        self.func(self.parent(),*self.args,**self.kwargs)
        self.quit()

class Project:
    """
        If a project file is provided (.mqpm):
            it loads that project
        elif name and folder are provided:
            Copies a template project to the provided folder.

            Creates a new MadQt project JSON
            file with project information.
            ProjectName.mqpm

        If no name nor folder is provided it just inits

        class Project

        Project(name='New MadQt project',folder,ico)

        Parameters:
            name - str
            folder - str
            ico - str


        Folder Structure

        self.folder
            *.mqpm
            *.sublime-project
            dev
                main.py
                *_rc.py
                gui
                    *.ui
                    *.py
                    *.qrc
                    *.img
                widgets
                    *.py
    """
    def __init__(self,name='',folder=None,icon=None):
        self.valid=False
        if '.mqpm' in name:
            self.loadSett(name)
        else:
            self.name = 'New MadQt project' if not name else name
            self.folder = folder
            if len(name) and folder:
                self.valid = self.create(icon)

    def create(self,icon):
        """creates a new project"""
        # Copy files in template folder to self.folder/dev
        source_dir = os.path.join(MadQt.get_path('Templates'),'NewProject')
        if os.path.isdir(self.devPath()):return False
        # copy template folder
        shutil.copytree(source_dir, self.devPath(), ignore=shutil.ignore_patterns('*.ico'))
        # copy .ico file
        shutil.copyfile(icon, self.guiFile('logo.ico'))
        if isTemp(icon): os.remove(icon)
        self.icon = 'logo.ico'

        # self.compileQrc('resources.qrc')
        # Create settings file
        self.settings = {
            'name':self.name,
            'folder':self.folder,
            'date':QDateTime.currentDateTime().toString(),
            'icon': 'logo.ico',
            'uiFiles':[],
            'customWidgets':{},
            'qrcFiles':['resources.qrc'],
        }
        self.saveSett()
        return True

    def compileQrc(self,qrc):
        """compiles a qrc file"""
        src = self.guiFile(qrc)
        Mt.compileQrc(src,self.devPath())

    def devPath(self):
        """returns dev folder"""
        return os.path.join(self.folder,'dev')

    def devFile(self,file):
            """returns file in dev path"""
            return os.path.join(self.devPath(),file)

    def guiFile(self,file):
            """returns file in gui path"""
            return os.path.join(self.devPath(),'gui',file)

    def widgetFile(self,file):
        """returns file in widgets path"""
        return os.path.join(self.devPath(),'widgets',file)

    def iconPath(self):
        """returns full path and filename for project icon"""
        return self.guiFile(self.icon)

    def setIcon(self,filename):
        self.icon = self.settings['icon'] = filename
        self.saveSett()

    def file(self):
        """returns full path and filename of madqt_project.mqpm """
        if not self.folder:return None
        return os.path.join(self.folder,f'{Mt.cleanString(self.name)}.mqpm')

    def saveSett(self):
        with open(self.file(), "w") as f:
            json.dump(self.settings, f, indent=4)

    def loadSett(self,file=None):
        file = self.file() if not file else file
        if os.path.isfile(file):
            with open(file, "r") as f:
                self.settings = json.loads(f.read())
            self.valid = True
            self.name = self.settings['name']
            self.icon = self.settings['icon']
            self.folder = self.settings['folder']
            self.date = self.settings['date']
        else:
            self.valid = False

    def runApp(self):
        """run main.py"""
        if not len(self.settings['uiFiles']):return None
        os.chdir(self.devPath())
        subprocess.Popen(['python', 'main.py'],shell=True)

    def saveToNewFolder(self,folder):
        if folder in self.folder:return False
        for file in os.listdir(self.folder):
            src = os.path.join(self.folder,file)
            dst = os.path.join(folder,file)
            if os.path.isdir(src):
                shutil.copytree(src, dst)
        self.settings['name']=self.name
        self.folder=self.settings['folder']=folder
        self.settings['date']=QDateTime.currentDateTime().toString()
        self.saveSett()
        return True

    def uiFiles(self):
        """returns generator with full path to ui files"""
        return (self.guiFile(ui) for ui in self.settings['uiFiles'])

    def qrcFiles(self):
        """returns generator with full path to qrc files"""
        return (self.guiFile(qrc) for qrc in self.settings['qrcFiles'])

    def customWidgetsFiles(self):
        """returns generator with full path to customWidgets module files"""
        return (self.widgetFile(cw) for cw in self.settings['customWidgets'].keys())

    def updateUi(self,ui_file):
        """
            Updates ui custom widgets and qrc files
            Compiles Qrc files
            Compiles py file
            Adds Ui class to compiled file:
                class Ui(QMainWindow):
                    def __init__(self):
                        super().__init__()
                        ui = Ui_MainWindow()
                        ui.setupUi(self)
        """
        ui_file = self.guiFile(ui_file)
        plugin_locations = MadQt_Designer_Paths
        widget_folder = os.path.join(self.devPath(),'widgets')
        plugin_locations.append(widget_folder)
        ui = Mt.Ui(ui_file,plugin_locations)

        # add qrc resources
        for qrc in self.settings['qrcFiles']: ui.add_qrc(qrc)

        # add promoted widgets
        for mod, classes in self.settings['customWidgets'].items():
            for cD in classes:
                ui.add_custom_widget(cD['class'],cD['qt_class'],mod.replace('.py','.h'))

        # save/overwrite ui to place qrc includes and custom widgets
        ui.save()

        # close to compile qrc files and a py
        # version that will work right off the box
        close_dict = ui.close()

        # Copy plugins that were found in our paths
        # but are not importable to widgets folder
        for plugin_mod in close_dict['in_paths']:
            py_name = plugin_mod+'.py'
            # already in widgets folder
            if os.path.isfile(self.widgetFile(py_name)):
                os.remove(self.widgetFile(py_name))
                # continue
            for path in plugin_locations:
                # don't check widgets folder
                if path == widget_folder: continue
                for file in os.listdir(path):
                    if file == py_name:
                        src = os.path.join(path,file)
                        dst = os.path.join(widget_folder,file)
                        shutil.copy2(src,dst)

        # add widget folder to widgets imports
        # from myWidgets >> from widgets.myWidgets
        # change resource imports
        # import resources_rc >> import gui.resources_rc
        from_widgets = close_dict['promoted_widgets']+close_dict['in_paths']
        for line in fileinput.input(close_dict['compiled_file'], inplace=1):
            for cw in from_widgets:
                if f'from {cw} import ' in line:
                    line = line.replace(cw, 'widgets.'+cw)

            for rs in self.settings['qrcFiles']:
                rs = rs.replace('.qrc','')
                if f'import {rs}' in line:
                    line = line.replace(rs, 'gui.'+rs)

            sys.stdout.write(line)

        # inform user of removed plugins and widgets
        if close_dict['removed_plugins']:

            msg = F"Missing plugins in '{os.path.basename(ui_file)}' \n\n"

            msg += F"The following plugins were removed: \n"
            for rp in close_dict['removed_plugins']:
                msg += F"- class:{rp['class']} | module:{rp['module']}\n"
            msg += F"As a consequence, widgets were removed: \n"
            for rw in close_dict['removed_widgets']:
                msg += F"- {rw['name']} \n"

            msg += F"\nQtDesigner plugins must be present either in\n"
            msg += F"Plugin Paths(in settings) or in the dev/widgets folder! \n\n"

            msg += F"To stop receiving this message: \n"
            msg += F"- ADD THE PLUGINS DIRECTORIES TO PLUGIN PATHS \n"
            msg += F"or\n"
            msg += F"- ADD MISSING PLUGINS TO PLUGIN/DEV/WIDGETS \n"
            msg += F"or\n"
            msg += F"- REMOVE MISSING PLUGINS FROM UI"
            reply = QMessageBox.information(
                window,
                "Could not import or find plugins!",
                msg
            )

    def addUi(self,file):
        """
        Copies file to dev/gui
        Adds custom widgets and qrc files to ui
        Compiles py file
        Adds ui to settings
        """
        # Copy file to dev/gui
        base = os.path.basename(file)
        dest = self.guiFile(base)
        shutil.copyfile(file, dest)

        if isTemp(file):os.remove(file)

        self.updateUi(base)

        self.settings['uiFiles'].append(base)
        self.saveSett()
        return dest

    def removeUi(self,file):
        """
        Removes ui
        Deletes ui file and compiled py file
        """
        os.remove(file)
        py_file = file.replace('.ui','.py')
        if os.path.isfile(py_file):os.remove(py_file)
        self.settings['uiFiles'].remove(os.path.basename(file))
        self.saveSett()

    def updateUis(self):
        """updates all UIs"""
        [self.updateUi(ui) for ui in self.uiFiles()]
        self.saveSett()

    def addQrc(self,basename):
        if basename in self.settings['qrcFiles']:return False
        self.settings['qrcFiles'].append(basename)
        self.updateUis()
        return True

    def createQrc(self,qrcName='resources.qrc'):
        """copies an empty qrc file from templates/Uis"""
        if qrcName in self.settings['qrcFiles']:
            pName,_=os.path.splitext(qrcName)
            qrcName = qrcName.replace(pName,pName+'_2')
        source_dir = os.path.join(MadQt.get_path('Templates'),'Uis','resources.qrc')
        shutil.copyfile(source_dir, self.guiFile(qrcName))
        self.addQrc(qrcName)
        return self.guiFile(qrcName)

class ProjectItem(QListWidgetItem):
    def __init__(self,file):
        self.file = file
        p = Project(file)
        ps = p.settings
        QListWidgetItem.__init__(self,QIcon(p.iconPath()),ps['name'])
        self.setToolTip(p.file())
        self.setStatusTip(f"Dbl click to open | Created: {ps['date']}")

class UiItem(QListWidgetItem):
    def __init__(self,file):
        self.file = file
        self.name = os.path.basename(file)
        QListWidgetItem.__init__(self,self.name)
        self.setStatusTip('Double click to open new QDesigner window or Drag and drop into QDesigner!')

class QrcFile(QTreeWidgetItem):
    def __init__(self,file,treeview):
        QTreeWidgetItem.__init__(self, treeview, type=0)
        self.file = file
        self.setText(0,os.path.basename(file))
        self.setExpanded(True)

class QrcPrefix(QTreeWidgetItem):
    def __init__(self,text,parent):
        QTreeWidgetItem.__init__(self, parent, type=1)
        self.setText(0,text)
        self.setExpanded(True)

class QrcImage(QTreeWidgetItem):
    def __init__(self,file,parent):
        QTreeWidgetItem.__init__(self, parent, type=2)
        self.file = file
        self.setText(0,os.path.basename(file))
        self.setIcon(0,QPixmap(file))
        self.setStatusTip(0,'Drag and drop into a image editor like Photoshop!')
        self.setExpanded(True)
        self.tinted = False

    def tint(self,color,_type):
        if not self.tinted:
            self.savedOriginal = os.path.join(tempDir(),os.path.basename(self.file))
            shutil.copy2(self.file,self.savedOriginal)
        Mt.tintImage(self.file,color,_type)
        self.setIcon(0,QPixmap(self.file))
        self.tinted = True

    def restore(self):
        shutil.copy2(self.savedOriginal,self.file)
        self.setIcon(0,QPixmap(self.file))

class CustomWidgetFile(QTreeWidgetItem):
    def __init__(self,file,parent):
        QTreeWidgetItem.__init__(self,parent, type=0)
        self.file = file
        self.setStatusTip(0,'Drag and drop into a text editor!')
        self.setText(0,os.path.basename(file))
        self.setExpanded(True)

class CustomWidgetClass(QTreeWidgetItem):
    def __init__(self,_class,sub_class,parent):
        QTreeWidgetItem.__init__(self,parent, type=1)
        self.setText(0,_class)
        self.setText(1,sub_class)
        self.setStatusTip(0,'Double click to open in sublime text!')
        self.setExpanded(True)

class App(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.setWindowFlags(Qt.FramelessWindowHint|Qt.WindowMaximizeButtonHint|Qt.WindowMinimizeButtonHint)
        self.setAttribute(Qt.WA_TranslucentBackground, True)
        self.ui = gui.Ui_MainWindow()
        self.ui.setupUi(self)
        self.selectedPath = None
        self.selectedFile = None
        self.extraMsgWidget=None
        self.threads = []
        self.prevGeo = self.geometry()
        # self.moved = False
        self.prevMainPageindex = 0
        self.project = Project() # current project loaded
        self.initSettings()
        self.initUi()
        self.readWindowSettings()

        # def pp():print(f"Hello")
        # self.act = QAction('Make tomatoes!')
        # self.act.triggered.connect(pp)
        # self.ui.runBtn.addAction(self.act)
        # self.act.trigger()
        # print(self.ui.runBtn.actions())
        # print(self.ui.actionPotatoes)

    # Ui Methods
    def thread(f):
        def wrapp(self, *a, **kw):
            return self.threads.append(ThreadF(self,f,*a,**kw))
        return wrapp

    def refresh(self):
        self.setCursor(Qt.WaitCursor)
        self.ui.statusbar.showMessage('Refreshing...')
        prevPF = self.project
        self.project = Project()
        self.openProject()
        self.project = prevPF
        self.openProject()
        self.ui.statusbar.showMessage('Project refreshed!')
        self.setCursor(Qt.ArrowCursor)

    def initSettings(self):
        self.settings = QSettings("Mad Pony Interactive", "MadQt | Project Manager")
        self.ui.NewPFolderName.setText(self.settings.value("usrInput/newProjectDir", None))
        # self.settings.remove('projects')
        # Populate Projects List
        allP = self.settings.value("projects",[])
        [self.addProject(p) for p in allP]

    def extraMsg(self,msg,timeout=5000):
        """adds extra message to StatusBar"""
        if self.extraMsgWidget!=None:self.ui.statusbar.removeWidget(self.extraMsgWidget)
        self.extraMsgWidget = QLabel(msg)
        self.extraMsgWidget.setObjectName('extraMsg')
        self.ui.statusbar.insertWidget(1,self.extraMsgWidget,1)
        QTimer.singleShot(timeout,self.removeExtraMsg)

    def removeExtraMsg(self):
        self.ui.statusbar.removeWidget(self.extraMsgWidget)
        self.extraMsgWidget=None

    def setMainPageIndex(self,i):
        """open previous page if i == None"""
        i = self.prevMainPageindex if i is None else i
        self.prevMainPageindex = self.ui.mainPage.currentIndex()
        if i == 0:
            self.ui.startPageBtn.setChecked(1)
        elif i == 2:
            self.ui.PpageBtn.setChecked(1)
        self.ui.mainPage.setCurrentIndex(i)

    def saveWindowSettings(self):
        self.settings.beginGroup("MainWindow")
        self.settings.setValue("geometry", self.saveGeometry())
        self.settings.setValue("windowState", self.saveState())
        self.settings.endGroup()

    def readWindowSettings(self):
        self.settings.beginGroup("MainWindow")
        geometry = self.settings.value("geometry", QByteArray())
        state = self.settings.value("windowState", QByteArray())
        if not geometry.isEmpty():
            self.restoreGeometry(geometry)
            self.restoreState(state)
        self.settings.endGroup()

    def closeEvent(self, event):
        self.saveWindowSettings()
        [thread.terminate() for thread in self.threads if thread.isRunning()]
        # delete temp files
        for file in os.listdir(tempDir()):
            file = os.path.join(tempDir(),file)
            if os.path.isfile(file):
                os.remove(file)
        event.accept()

    def eventFilter(self, obj, event):
        # print(event.type())
        if obj.objectName() == 'header':
            if self.ui.logo.underMouse():
                if event.type() == QEvent.MouseButtonRelease:
                    webbrowser.open('https://madponyinteractive.github.io/MadQt/')
                    return True
            else:
                if event.type() == QEvent.MouseButtonDblClick:
                    self.setWindowState(self.windowState() ^ Qt.WindowFullScreen)
                    return True

                if event.type() == QEvent.MouseButtonRelease:
                    if event.globalPosition().y() < 10 and self.moved:
                        self.prevGeo = self.geometry()
                        self.showMaximized()
                        return True

                if event.type() == QEvent.MouseButtonPress:
                    self.prevMousePos = event.scenePosition()
                    self.moved = False

                if event.type() == QEvent.MouseMove:
                    if self.windowState() == Qt.WindowFullScreen\
                    or self.windowState() == Qt.WindowMaximized:
                        self.showNormal()
                        self.prevMousePos = QPointF(self.prevGeo.width()*.5,50)

                    gr=self.geometry()
                    screenPos = event.globalPosition()
                    pos = screenPos-self.prevMousePos
                    x = max(pos.x(),0)
                    y = max(pos.y(),0)
                    screen = QGuiApplication.screenAt(QPoint(x,y)).size()
                    x = min(x,screen.width()-gr.width())
                    y = min(y,screen.height()-gr.height())

                    self.move(x,y)
                    self.moved = True

                    # print(QGuiApplication.screens())
        return QMainWindow.eventFilter(self, obj, event)

    '''
    # A play around with custom resize handles
    def event(self, event):
        if event.type() == QEvent.MouseButtonRelease:
            self.grabbedHandle = False

        if event.type() == QEvent.MouseMove:
            if self.handle and event.buttons()==Qt.LeftButton:
                self.grabbedHandle=True
                gr=self.geometry()
                gp=event.globalPosition()
                minW=self.minimumSizeHint().width()
                minH=self.minimumSizeHint().height()
                x = int(gp.x())
                y = int(gp.y())
                # x = max(x,minW)
                if self.handle == 'top':
                    # y = min(y,minH)
                    gr.setTop(y)
                    # self.move(self.x(),y)
                elif self.handle == 'bottom':
                    gr.setBottom(y)



                # self.resize(gr.size())
                self.setGeometry(gr)
                self.updateGeometry()

        if event.type() == QEvent.HoverMove and not self.grabbedHandle:
            x=int(event.position().x())
            y=int(event.position().y())
            gr=self.geometry()
            w=gr.width()
            h=gr.height()
            o=8
            yr=y in range(o,h-o)
            xr=x in range(o,w-o)
            if not yr and not xr:
                # corners
                if y<o and x<o:
                    self.handle ='top left'
                    self.setCursor(Qt.SizeFDiagCursor)
                elif x>w-o and y<o:
                    self.handle ='top right'
                    self.setCursor(Qt.SizeBDiagCursor)
                elif x<o and y>h-o:
                    self.handle ='bottom left'
                    self.setCursor(Qt.SizeBDiagCursor)
                elif x>w-o and y>h-o:
                    self.handle ='bottom right'
                    self.setCursor(Qt.SizeFDiagCursor)
                else: self.handle = None
            else:
                # edges
                if x<o and yr:
                    self.handle ='left'
                    self.setCursor(Qt.SizeHorCursor)
                elif x>w-o and yr:
                    self.handle ='right'
                    self.setCursor(Qt.SizeHorCursor)
                elif y<o and xr:
                    self.handle ='top'
                    self.setCursor(Qt.SizeVerCursor)
                elif y>h-o and xr:
                    self.handle ='bottom'
                    self.setCursor(Qt.SizeVerCursor)
                else: self.handle = None

            if self.handle:
                pass
            else:
                self.setCursor(Qt.ArrowCursor)

        return QMainWindow.event(self, event)
    '''

    def initUi(self):
        self.ui.header.installEventFilter(self)
        self.ui.statusbar.insertPermanentWidget(0,QLabel('V.0.0.2'),0)

        # Start Page
        self.ui.startPageBtn.clicked.connect(lambda: self.setMainPageIndex(0))
        self.ui.newP.clicked.connect(lambda: self.setMainPageIndex(1))
        self.ui.addP.clicked.connect(self.addProjectFromFile)
        self.ui.delSelectedP.clicked.connect(lambda: self.removeSelectedProjects())
        self.ui.settBtn.clicked.connect(lambda: self.setMainPageIndex(3))
        self.ui.PpageBtn.clicked.connect(lambda: self.setMainPageIndex(2))
        self.ui.ProjectsList.itemDoubleClicked.connect(self.openProject)
        self.ui.ProjectsList.dropped.connect(self.droppedProject)

        # Settings Page
        default_qtPath = os.path.join(pySideDir())
        # self.settings.setValue("usrInput/designerPath", default_qtPath)
        self.ui.QtDesignerPathInput.setText(self.settings.value("usrInput/designerPath", default_qtPath))
        self.ui.sublimePathInput.setText(self.settings.value("usrInput/sublimePath", ''))

        self.ui.icoSizeCb.setCurrentText(self.settings.value("usrInput/icoSize",str(64)))
        def changedIco(v): self.settings.setValue("usrInput/icoSize",v)
        self.ui.icoSizeCb.currentTextChanged.connect(changedIco)

        self.ui.QtDesignerPathBtn.clicked.connect(lambda: self.openFolder(self.ui.QtDesignerPathInput))
        self.ui.sublimePathBtn.clicked.connect(lambda: self.openFolder(self.ui.sublimePathInput))

        global MadQt_Designer_Paths
        MadQt_Designer_Paths = self.settings.value("usrInput/pluginPaths", [MadQt.get_path('QtDesignerPlugins')])
        for item in MadQt_Designer_Paths:
            self.ui.designerPathsList.addItem(item)

        self.ui.SettDoneBtn.clicked.connect(lambda: self.setMainPageIndex(None))
        self.ui.saveSett.clicked.connect(self.saveUserSettings)

        self.ui.createMQEXEC.clicked.connect(lambda: webbrowser.open("https://madponyinteractive.github.io/MadQt/ProjectManager/install.html"))

        # New custom widget page
        self.ui.cwQtClass.addItems(Mt.QDesignerBaseClasses())
        self.ui.cwQtClass.setCurrentText('QPushButton')
        self.ui.createWidget.clicked.connect(self.createWidget)
        self.ui.cancelCreateW.clicked.connect(lambda: self.setMainPageIndex(None))

        # New project page
        self.ui.newPFolder.clicked.connect(lambda: self.openFolder(self.ui.NewPFolderName))
        self.ui.newPIconBrowse.clicked.connect(lambda: self.openImg(self.ui.newPIcon,True))
        self.ui.createP.clicked.connect(self.createNewProject)
        self.ui.cancelCreateP.clicked.connect(lambda: self.setMainPageIndex(None))

        # New Ui Page
        self.ui.createNewUi.clicked.connect(self.createUi)
        self.ui.cancelCreateUi.clicked.connect(lambda: self.setMainPageIndex(None))

        # Project Page menu buttons
        self.ui.refreshBtn.clicked.connect(self.refresh)
        self.ui.openPFolder.clicked.connect(self.openProjectFolder)
        if self.hasSublime():
            self.ui.sublimePBtn.setStatusTip('Open sublime project')
        else:
            self.ui.sublimePBtn.setStatusTip('Setup sublime path in settings to open sublime projects')

        self.ui.sublimePBtn.clicked.connect(self.openSublimeProject)
        self.ui.uiFilesBtn.clicked.connect(lambda: self.ui.PStackedWidget.setCurrentIndex(0))
        self.ui.custWidgetsBtn.clicked.connect(lambda: self.ui.PStackedWidget.setCurrentIndex(1))
        self.ui.qrcPBtn.clicked.connect(lambda: self.ui.PStackedWidget.setCurrentIndex(2))
        self.ui.execPBtn.clicked.connect(lambda: self.ui.PStackedWidget.setCurrentIndex(3))
        self.ui.runBtn.clicked.connect(self.runApp)
        self.ui.saveAsBtn.clicked.connect(self.saveToNewFolder)

        # Ui page
        self.ui.uiList.itemDoubleClicked.connect(self.openUi)
        self.ui.uiList.dropped.connect(self.addUiFiles)
        self.ui.addUiBtn.clicked.connect(self.addUiFiles)
        self.ui.delSelectedUiBtn.clicked.connect(self.removeUi)
        self.ui.newUiBtn.clicked.connect(lambda: self.setMainPageIndex(5))

        # Custom Widget page
        self.ui.moduleTree.header().setSectionResizeMode(QHeaderView.Stretch)
        self.ui.moduleTree.droppedModule.connect(self.addModules)
        self.ui.moduleTree.itemDoubleClicked.connect(self.openClass)
        self.ui.addModuleBtn.clicked.connect(self.addModuleDialogue)
        self.ui.delSelectedMods.clicked.connect(self.deleteModules)
        self.ui.newModuleBtn.clicked.connect(self.newModule)
        self.ui.cwClass.textChanged.connect(self.cwClassChanged)
        self.ui.cwMod.textChanged.connect(self.cwModChanged)
        self.ui.cwQtClass.currentTextChanged.connect(self.cwQtClassChanged)

        # Qrc Image page
        self.ui.qrcTree.header().setSectionResizeMode(QHeaderView.Stretch)
        self.ui.qrcTree.droppedQrc.connect(self.addQrc)
        self.ui.qrcTree.droppedImg.connect(self.addImages)

        iSize = self.settings.value("usrInput/iconSize",QSize(64,64))
        self.ui.qrcTree.setIconSize(iSize)
        self.ui.imgSizeSlider.setValue(iSize.width())
        def setIconSize(s):
            size=QSize(s,s)
            self.ui.qrcTree.setIconSize(size)
            self.settings.setValue("usrInput/iconSize",size)
            self.ui.statusbar.showMessage(f'Icon size: {s}px')
        self.ui.imgSizeSlider.valueChanged.connect(setIconSize)

        # Create Executable page
        self.ui.createExecBtn.clicked.connect(self.createExec)
        self.ui.pyinstallerHelp.clicked.connect(lambda:\
         webbrowser.open('https://pyinstaller.readthedocs.io/en/stable/when-things-go-wrong.html'))

    # Create Executable
    @thread
    def runExec(self,arguments):
        process = subprocess.Popen(arguments, shell=True, bufsize = 1,
            stdout=subprocess.PIPE, stderr = subprocess.STDOUT,
            encoding='utf-8', errors = 'replace')
        while True:
            realtime_output = process.stdout.readline()
            if realtime_output == '' and process.poll() is not None:
                self.finishExec()
                break
            if realtime_output:
                self.ui.execOutput.append(realtime_output.strip())

    def finishExec(self):
        for file in os.listdir(self.project.devPath()):
            if os.path.splitext(file)[1] == '.spec':
                os.remove(file)
                break
        shutil.rmtree(self.project.devFile('build'))
        src = self.project.devFile('dist')
        dst = self.project.folder

        findAt = os.path.join(dst,'dist')
        if os.path.isdir(findAt):shutil.rmtree(findAt)
        shutil.move(src, dst, copy_function = shutil.copytree)

        self.ui.execOutput.append(F"Successfully created executable for project: {self.project.name}!")
        self.ui.execOutput.append(F"Output files located @ {findAt}")
        self.ui.execOutput.moveCursor(QTextCursor.End)

        self.ui.statusbar.showMessage('Executable created!')

    def createExec(self):
        self.ui.statusbar.showMessage('Creating executable...')
        self.project.updateUis()
        os.chdir(self.project.devPath())
        arguments = F"pyinstaller "
        arguments += self.ui.execArgs.text()
        arguments +=F" --icon {os.path.join('gui',self.project.icon)} main.py"
        self.ui.execOutput.ensureCursorVisible()
        self.ui.execOutput.append(f'Running: {arguments}')
        self.runExec(arguments)

    # Helper Dialogues
    def openFile(self,fType='All Files (*)',prefix=''):
        return QFileDialog.getOpenFileName(self,
                "MadQt Project Manager",
                prefix,
                fType, "")

    def openFiles(self,fType='All Files (*)',prefix=''):
        return QFileDialog.getOpenFileNames(self,
                "MadQt Project Manager",
                prefix,
                fType, "")

    def openFolder(self,widget):
        """ Opens a file dialogue
            Sets text on provided widget
        """
        text = widget.text() if len(widget.text()) else ''
        options = QFileDialog.DontResolveSymlinks | QFileDialog.ShowDirsOnly
        directory = QFileDialog.getExistingDirectory(self,
                "MadQt Project Manager - Select folder",
                text, options)
        if directory:
            widget.setText(directory)
            self.selectedPath = directory

    def openImg(self,widget,ico=False):
        """ Opens a file dialogue
            Sets pixmap on provided widget
        """
        file = self.openFile('Images (*.png *.ico *.jpg *.gif *.svg)',widget.text())[0]
        if len(file):
            if ico:
                if '.ico' not in file:
                    dest_ico = os.path.join(tempDir(),'logo.ico')
                    file = Mt.createIco(file,dest_ico,self.defaultIcoSize())
            widget.setPixmap(QPixmap(file))
            self.selectedFile = file
            # self.settings.setValue("usrInput/newProjectIcon", directory)

    def questionBox(self,msg):
        reply = QMessageBox.question(self, "MadQt Warning!",
                msg,
                QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
        if reply == QMessageBox.Yes:
            return "Yes"
        elif reply == QMessageBox.No:
            return "No"
        else:
            return "Cancel"

    def labelBox(self,msg,text):
        usrInput, ok = QInputDialog.getText(self, "MadQt Project Manager",
            msg, QLineEdit.Normal, text)
        if ok and usrInput != '':
            return usrInput

    # Project Methods
    def addProject(self,file=None,save=False):
        """
            Adds item to the start page project list
            Saves project to settings if save=True
        """
        if isinstance(file,QListWidgetItem):
            file=file.file
            save=True
        elif not file:
            file = self.project.file()

        # File was deleted from system?
        if not os.path.isfile(file):
            self.removeProject(file,True)
            return

        # File already added?
        for item in self.ui.ProjectsList.getItems():
            if item.file == file:
                # self.removeProject(file,True)
                self.extraMsg('Project folder already in list!')
                return

        # Add project to the start page project list
        self.ui.ProjectsList.insertItem(0,ProjectItem(file))
        # Save project to settings
        if not save:return
        allP = self.settings.value("projects",[])
        # remove projects with same folder
        [self.removeProject(p,True) for p in allP if p == file]
        allP.append(file)
        self.settings.setValue("projects", allP)

    def removeProject(self,file=None,save=False):
        """
            Removes item from the start page project list
            Removes project from settings if save=True
        """
        if isinstance(file,QListWidgetItem):
            file=file.file
            save=True
        elif not file:
            file = self.project.file()

        self.ui.ProjectsList.removeItem(file)

        # Removes project from settings
        if not save:return
        allP = self.settings.value("projects",[])
        [allP.remove(p) for p in allP if p == file]
        self.settings.setValue("projects", allP)
        if not len(allP):self.ui.PpageBtn.setEnabled(False)

    def removeSelectedProjects(self):
        """removes all selected projects in project list"""
        [self.removeProject(item) for item in self.ui.ProjectsList.selectedItems()]

    def addProjectFromFile(self,file=None):
        """Add and open a project from an existing file"""
        file = self.openFile('MadQt Project Files (*.mqpm)')[0]\
         if not file else file
        if not len(file):return

        if not file.endswith('mqpm'):
            self.extraMsg('Not a valid project file!')
            return

        # Update folder path
        prevSettings = {}
        with open(file, "r") as f: prevSettings = json.loads(f.read())
        prevSettings['folder'] = os.path.dirname(file)
        with open(file, "w") as f: json.dump(prevSettings, f, indent=4)

        self.addProject(file,True)

    def createNewProject(self):
        name = self.ui.projectNameInput.text()
        folder = self.ui.NewPFolderName.text()
        if not os.path.isdir(folder):
            self.ui.statusbar.showMessage('Provided path not found!',5000)
            return
        name = 'New MadQt project' if not name else name.strip()
        self.settings.setValue("usrInput/newProjectDir", folder)

        folder = os.path.join(folder,Mt.cleanString(name))

        ico = os.path.join(MadQt.get_path('Templates'),'NewProject','gui','logo.ico')
        if self.selectedFile is not None:
            ico = ico if '.ico' not in self.selectedFile else self.selectedFile
        self.project = Project(name,folder,ico)

        if not self.project.valid:
            self.project = Project()
            self.ui.statusbar.showMessage('Could not create project! Provided folder has a project?')
            return

        # self.settings.remove('projects')#333333333333333333333333333333333
        self.addProject(save=True)
        self.extraMsg(f'Successfully created "{name}"!')
        self.openProject()

    def saveToNewFolder(self):
        """open file dialogue"""
        options = QFileDialog.DontResolveSymlinks | QFileDialog.ShowDirsOnly
        fileName, filtr = QFileDialog.getSaveFileName(self,
                "MadQt Project Manager - Save to new folder",
                self.project.folder,
                "MadQt Project Files (*.mqpm)", "", options)

        file_name = os.path.basename(fileName)
        file, ext = os.path.splitext(file_name)
        directory = os.path.dirname(fileName)

        if fileName != '':
            self.project.name = file
            if self.project.saveToNewFolder(directory):
                self.addProject(save=True)
                self.extraMsg(f'Successfully saved and opened!')
                self.openProject()
            else:
                self.extraMsg(f'Could not save, saving in a project folder?')

    def droppedProject(self, urls):
        """when user drops file on projects list"""
        [self.addProjectFromFile(file) for file in urls]

    def openProject(self,item=None):
        """open existing project"""
        if item:
            # Avoid re-opening same project
            if item.file == self.project.file():
                self.setMainPageIndex(2)
                return
            self.project = Project(item.file)
        self.setCursor(Qt.WaitCursor)

        self.ui.qrcTree.clear()
        self.ui.moduleTree.clear()
        self.ui.uiList.clear()
        if not self.project.valid:return

        self.ui.statusbar.showMessage('Loading project...')

        self.ui.PpageBtn.setEnabled(True)
        if self.hasSublime(): self.ui.sublimePBtn.setEnabled(True)
        self.ui.PpageBtn.setChecked(True)
        self.ui.PrjName.setText(self.project.name)

        self.project.updateUis()

        # Populate Ui list
        [self.addUi(ui) for ui in self.project.uiFiles() if os.path.isfile(ui)]
        # Populate Qrc list
        [self.addQrc(qrc) for qrc in self.project.qrcFiles() if os.path.isfile(qrc)]
        # Populate Custom Widget list
        [self.addModule(mod) for mod in self.project.customWidgetsFiles() if os.path.isfile(mod)]

        self.ui.execArgs.setText(F"--onefile --windowed --name={Mt.cleanString(self.project.name)}")

        self.setMainPageIndex(2)
        self.ui.statusbar.showMessage(f'{self.project.name} project loaded!')
        self.setCursor(Qt.ArrowCursor)

    def openProjectFolder(self):
        # Open base on project page
        Mt.openFileExplorer(self.project.folder)

    # Qrc Methods
    def createQrc(self):
        qrcName = self.labelBox("New Qrc name:",'resources.qrc')
        if not qrcName:return
        self.setCursor(Qt.WaitCursor)
        if '.' not in qrcName: qrcName = qrcName + '.qrc'
        fn,ext=os.path.splitext(qrcName)
        qrcName=Mt.cleanString(fn,True)+ext
        qrcFile = self.project.createQrc(qrcName)
        self.addQrc(qrcFile)
        self.setCursor(Qt.ArrowCursor)

    def addQrc(self,file=None,save=False):
        """
            Adds qrc files, prefixes and images
            to the project qrc tree
            Saves qrc to project settings if save=True
        """
        if file is None:# file dialogue
            qrcs = self.openFiles('Qrc Resource files (*.qrc)')[0]
            [self.addQrc(qrc,True) for qrc in qrcs]
            return
        if isinstance(file,list):# dropped
            [self.addQrc(qrc,True) for qrc in file]
            return

        if save:
            base = os.path.basename(file)
            if not self.project.addQrc(base):
                self.extraMsg(f"{base} not added, a file with the same name already exists in project!")
                return

        res_item=QrcFile(file,self.ui.qrcTree)
        parsed = xml.parse(file)
        for resource in parsed.findall('qresource'):
            # print(resource.get('prefix'))
            pre_item=self.addPrefix(resource.get('prefix'),res_item)
            for img in resource.findall('file'):
                self.addImage(self.project.guiFile(img.text),pre_item,file)
                # print(img.text)
        return res_item
        # self.ui.qrcTree.addTopLevelItem()

    def addPrefix(self,prefix=None,parent=None,save=False,add=True):
        qrc=None
        if not prefix:# QLineEdit popup
            msg = ''
            # add=self.ui.addPrefixBtn.text()=='Add Prefix'
            if add:
                placeholder='newPrefix'
                topLevel = parent = self.ui.qrcTree.topLevelItem(0)
                if topLevel:# not empty
                    items = self.ui.qrcTree.selectedItems()
                    if not len(items):
                        qrc = topLevel.text(0)
                        msg+='No qrc file selected, will add prefix to ' + qrc
                    else:
                        foundQrcs=[]
                        [foundQrcs.append(item) for item in items if item.type()==0]
                        if len(foundQrcs)>1:
                            qrc = topLevel.text(0)
                            msg+='Multiple qrc files selected, will add prefix to ' + qrc
                        else:
                            parent = foundQrcs[0]
                            qrc = parent.text(0)
                    qrc = self.project.guiFile(qrc)# full path
                else:
                    msg +="This will create a resources.qrc file!"
                msg +="\nPrefix name:"
            else:# rename
                placeholder=self.ui.qrcTree.currentItem().text(0)
                msg +="WARNING:\n"
                msg +="When renaming prefixes your ui paths do not get updated!\n"
                msg +="If you rename a prefix that is already in use by an ui,\n"
                msg +="you will need to update the ui resource path to display its images.\n"
                msg +="\n New Prefix name:"

            prefix, ok = QInputDialog.getText(self, "MadQt Project Manager",
                    msg, QLineEdit.Normal,
                    placeholder)
            if ok and prefix != '':
                if add:
                    if not qrc:
                        qrc = self.project.createQrc()
                        parent = self.addQrc(qrc)
                    save=True
                else:# rename
                    current = self.ui.qrcTree.currentItem()
                    prevPrefix = current.text(0)
                    current.setText(0,prefix)
                    qrc = current.parent().file
                    parsed = xml.parse(qrc)
                    for res in parsed.findall('qresource'):
                        if res.get('prefix')==prevPrefix:
                            res.set('prefix',prefix)
                            break
                    xml.indent(parsed.getroot())
                    parsed.write(qrc)
                    return

        if save:
            # add prefix to qrc file
            if not qrc: qrc = parent.file
            parsed = xml.parse(qrc)
            pfE = xml.Element('qresource')
            pfE.set('prefix',prefix)
            parsed.getroot().append(pfE)
            self.project.updateUis()
            xml.indent(parsed.getroot())
            parsed.write(qrc)
            self.ui.statusbar.showMessage('Done')

        return QrcPrefix(prefix,parent)

    def addImagesDialogue(self):
        imgs = self.openFiles('Images (*.png *.ico *.jpg *.gif *.svg)')[0]
        if len(imgs):self.addImages(imgs)

    @thread
    def addImages(self,imgs=None):
        if isinstance(imgs,dict):# dropped
            [self.addImage(img,imgs['prefix'],imgs['qrc'],True) for img in imgs['imgs']]
        else:# file dialogue
            prefix=None
            qrc = self.ui.qrcTree.topLevelItem(0)# could be None
            if qrc:# not empty
                items = self.ui.qrcTree.selectedItems()
                if not len(items):# nothing selected
                    prefix = qrc.child(0)# could be None
                else:
                    current = self.ui.qrcTree.currentItem()
                    if current.type()==2:# current image
                        prefix = current.parent()
                        qrc = current.parent().parent()
                    elif current.type()==1:# current prefix
                        prefix=current
                        qrc=current.parent()
                    else:# current qrc
                        qrc=current
            [self.addImage(img,prefix,qrc,True) for img in imgs]

    def addImage(self,img,prefix=None,qrc=None,save=False):
        """
            adds image to qrcTree
            adds to qrc file if save
        """
        if save:
            self.ui.statusbar.showMessage(f'Saving {img}...')
            if not qrc:
                qrc = self.project.createQrc()
                qrc = self.addQrc(qrc)
            if not prefix:
                prefix = self.addPrefix('newPrefix',qrc,True)

            # add image file to qrc file
            parsed = xml.parse(qrc.file)
            fE = xml.Element('file')
            fE.text = os.path.basename(img)
            for res in parsed.findall('qresource'):
                if res.get('prefix') == prefix.text(0):
                    res.append(fE)

            xml.indent(parsed.getroot())
            parsed.write(qrc.file)

            dest_img = self.project.guiFile(fE.text)
            shutil.copy2(img,dest_img)
            img=dest_img

            self.project.updateUis()
            self.ui.statusbar.showMessage('Done')

        QrcImage(img,prefix)

    def tintImagesDlg(self,imgs,_type):
        """tint selected images dialogue"""
        c = QColorDialog.getColor(self.settings.value("usrInput/tintColor",Qt.red), self)
        if c.isValid():
            self.settings.setValue("usrInput/tintColor", c)
            self.tintImages(imgs,c,_type)

    @thread
    def tintImages(self,imgs,c,_type):
        """tint selected images"""
        [img.tint((c.red(),c.green(),c.blue()),_type) for img in imgs]

    @thread
    def restoreImages(self,imgs):
        """undo tinted selected images"""
        [img.restore() for img in imgs if img.tinted]

    @thread
    def makeIco(self,imgs):
        for img in imgs:
            if '.ico' not in img.file:
                base = os.path.basename(img.file)
                fname,_=os.path.splitext(base)
                dest_ico = os.path.join(tempDir(),fname+'.ico')
                Mt.createIco(img.file,dest_ico,self.defaultIcoSize())
                self.addImage(dest_ico,img.parent(),img.parent().parent(),True)

    @thread
    def setIcon(self,img):
        """Sets current image as project icon"""
        base = os.path.basename(img.file)
        self.project.setIcon(base)
        for project in self.ui.ProjectsList.getItems():
            if project.file == self.project.file():
                project.setIcon(QPixmap(img.file))
                break

    def renameImage(self,item):
        fName,fExt=os.path.splitext(item.text(0))
        usrInput = self.labelBox("New Image name:",fName)
        if usrInput:
            if '.' in usrInput:
                usrInput = usrInput.replace(os.path.splitext(usrInput)[1],'')
            usrInput=Mt.cleanString(usrInput,True)+fExt
            qrc = item.parent().parent().file
            prefix = item.parent().text(0)
            dest = item.file.replace(item.text(0),usrInput)
            os.rename(item.file, dest)
            # project icon?
            if self.project.iconPath() == item.file:self.setIcon(item)
            item.file = dest
            parsed = xml.parse(qrc)
            for res in parsed.findall('qresource'):
                if res.get('prefix')==prefix:
                    for file in res.findall('file'):
                        if file.text == item.text(0):
                            file.text=usrInput
                            break
            item.setText(0,usrInput)
            xml.indent(parsed.getroot())
            parsed.write(qrc)
            self.project.updateUis()

    def renameQrc(self,item):
        fName,fExt=os.path.splitext(item.text(0))
        usrInput = self.labelBox("New Qrc name:",fName)
        if usrInput:
            if '.' in usrInput:
                usrInput = usrInput.replace(os.path.splitext(usrInput)[1],'')
            usrInput=Mt.cleanString(usrInput,True)+fExt

            dest = item.file.replace(item.text(0),usrInput)
            os.rename(item.file, dest)
            item.file = dest
            self.project.settings['qrcFiles'].remove(item.text(0))
            self.project.settings['qrcFiles'].append(usrInput)
            item.setText(0,usrInput)
            self.project.updateUis()

    def deleteQrcs(self,items):
        msg ="This will delete the selected qrc files and all it's images!\n"
        msg +="Do you wish to proceed?"
        answer = self.questionBox(msg)
        if answer != 'Yes':return
        [self.deleteQrc(i) for i in items]

    @thread
    def deleteQrc(self,item):
        self.ui.statusbar.showMessage(f'Deleting Qrc: {item.text(0)}...')
        parsed = xml.parse(item.file)
        for resource in parsed.findall('qresource'):
            for image in resource.findall('file'):
                imgPath = self.project.guiFile(image.text)
                os.remove(imgPath)
        os.remove(item.file)
        self.project.settings['qrcFiles'].remove(item.text(0))
        self.project.updateUis()
        self.ui.qrcTree.removeTopLevelItem(item)
        self.ui.statusbar.showMessage('Done')

    def deletePrefixes(self,items):
        msg ="This will delete the selected prefixes and all it's images!\n"
        msg +="Do you wish to proceed?"
        answer = self.questionBox(msg)
        if answer != 'Yes':return
        [self.deletePrefix(i) for i in items]

    @thread
    def deletePrefix(self,item):
        self.ui.statusbar.showMessage(f'Deleting Prefix: {item.text(0)}...')
        parsed = xml.parse(item.parent().file)
        for resource in parsed.findall('qresource'):
            if resource.get('prefix')==item.text(0):
                for image in resource.findall('file'):
                    imgPath = self.project.guiFile(image.text)
                    os.remove(imgPath)
                parsed.getroot().remove(resource)

        xml.indent(parsed.getroot())
        parsed.write(item.parent().file)
        self.project.updateUis()
        item.parent().removeChild(item)
        self.ui.statusbar.showMessage('Done')

    def deleteImages(self,items):
        msg ="This will permanently delete the selected images!\n"
        msg +="Do you wish to proceed?"
        answer = self.questionBox(msg)
        if answer != 'Yes':return
        [self.deleteImage(i) for i in items]

    @thread
    def deleteImage(self,item):
        # project icon?
        if self.project.iconPath() == item.file:
            self.ui.statusbar.showMessage(f'Image {item.text(0)} is project icon and cannot be deleted!')
            return
        self.ui.statusbar.showMessage(f'Deleting Image: {item.text(0)}...')
        parsed = xml.parse(item.parent().parent().file)
        for resource in parsed.findall('qresource'):
            if resource.get('prefix')==item.parent().text(0):
                for image in resource.findall('file'):
                    if image.text==item.text(0):
                        imgPath = self.project.guiFile(image.text)
                        os.remove(imgPath)
                        resource.remove(image)
        xml.indent(parsed.getroot())
        parsed.write(item.parent().parent().file)
        self.project.updateUis()
        item.parent().removeChild(item)
        self.ui.statusbar.showMessage('Done')

    # Modules methods (Custom Widgets)
    def deleteModules(self):
        items=self.ui.moduleTree.selectedModules()
        msg ="This will permanently delete the selected modules!\n"
        msg +="Do you wish to proceed?"
        answer = self.questionBox(msg)
        if answer != 'Yes':return
        [self.deleteModule(i) for i in items]

    @thread
    def deleteModule(self,item):
        self.ui.statusbar.showMessage(f'Deleting Module: {item.text(0)}...')
        if os.path.isfile(item.file):os.remove(item.file)
        self.project.settings['customWidgets'].pop(item.text(0))
        self.project.updateUis()
        self.ui.moduleTree.removeTopLevelItem(item)
        self.ui.statusbar.showMessage('Done')

    def renameModule(self,item):
        fName,fExt=os.path.splitext(item.text(0))
        usrInput = self.labelBox("New Module name:",fName)
        if usrInput:
            if '.' in usrInput:
                usrInput = usrInput.replace(os.path.splitext(usrInput)[1],'')
            usrInput=Mt.cleanString(usrInput,True)+fExt

            dest = item.file.replace(item.text(0),usrInput)
            os.rename(item.file, dest)
            item.file = dest
            moduleValue = self.project.settings['customWidgets'].get(item.text(0))
            self.project.settings['customWidgets'].pop(item.text(0))
            self.project.settings['customWidgets'][usrInput] = moduleValue
            item.setText(0,usrInput)
            self.project.updateUis()

    def addModules(self,mods=None):
        if not mods: mods = self.openFiles('Custom Widgets (*.py)')[0]
        [self.addModule(mod,True) for mod in mods]

    def addModuleDialogue(self):
        mods = self.openFiles('Custom Widgets (*.py)')[0]
        if len(mods):self.addModules(mods)

    @thread
    def addModule(self,file=None,save=False):
        basename = os.path.basename(file)
        if save:
            if self.project.settings['customWidgets'].get(basename,False):
                self.ui.statusbar.showMessage(f'A custom widgets named: {basename} already exists!')
                return
            self.project.settings['customWidgets'][basename]=[]
            src=file
            file = self.project.widgetFile(basename)
            shutil.copy2(src,file)
        moduleItem = CustomWidgetFile(file, self.ui.moduleTree)
        valueList = []

        with open(file,"r") as f:
            for line in f:
                if 'class' in line:
                    qtClass = [_class for _class in Mt.QDesignerBaseClasses() if _class in line]
                    if len(qtClass):
                        start = line.index('class ')+6
                        end = line.index('(')
                        userClass = line[start:end]
                        CustomWidgetClass(userClass, qtClass[0], moduleItem)
                        valueList.append({'class':userClass,'qt_class':qtClass[0]})
        self.project.settings['customWidgets'][basename]=valueList
        self.project.saveSett()
        if save:
            self.project.updateUis()

    def openModule(self,item):
        """opens in sublime text"""
        # print(item.file)
        if not self.hasSublime():return
        os.chdir(self.ui.sublimePathInput.text())
        subprocess.Popen(['subl', item.file],shell=True)

    def openClass(self,item,col=None):
        """opens in sublime text"""
        if isinstance(item,CustomWidgetFile):return
        if not self.hasSublime():
            self.ui.statusbar.showMessage('Sublime Text path not set. Navigate to Settings to set it.')
            return
        lineToOpen=0
        with open(item.parent().file,"r") as f:
            for i, line in enumerate(f):
                if item.text(0) in line:
                    lineToOpen=str(i+1)
                    break
        os.chdir(self.ui.sublimePathInput.text())
        subprocess.Popen(['subl', item.parent().file+':'+lineToOpen],shell=True)

    def addClass(self):
        moduleItem = self.ui.moduleTree.currentItem()
        self.ui.cwModLbl.hide()
        self.ui.cwMod.hide()
        self.ui.cwMod.setText(moduleItem.text(0)[:-2])
        self.setMainPageIndex(4)

    def newModule(self):
        self.ui.cwModLbl.show()
        self.ui.cwMod.show()
        self.setMainPageIndex(4)

    def cwModChanged(self,text):
        if len(text):
            text = Mt.cleanString(text,True)
            self.ui.cwMod.setText(text[0].lower() + text[1:])

    def cwQtClassChanged(self,text):
        self.ui.cwClass.setPlaceholderText(text[1:])
        if self.ui.cwMod.isVisible():
            text = text[1:]
            text = text[0].lower() + text[1:]
            self.ui.cwMod.setPlaceholderText(text)

    def cwClassChanged(self,text):
        if len(text):
            text = Mt.cleanString(text,True)
            self.ui.cwClass.setText(text[0].upper() + text[1:])
            self.ui.cwMod.setPlaceholderText(text[0].lower() + text[1:])

    def createWidget(self):
        qt_class = self.ui.cwQtClass.currentText()

        _class = self.ui.cwClass.text()
        _class = _class if _class != '' else self.ui.cwClass.placeholderText()

        module = self.ui.cwMod.text()
        module = module if module != '' else self.ui.cwMod.placeholderText()

        filename = module +'.py'

        if self.ui.cwMod.isVisible():# new module
            file = os.path.join(tempDir(),filename)
            with open(file, "w") as fp:
                fp.write(customWidgetModule(_class,qt_class))
            self.addModule(file,True)
        else:# add class
            file = self.project.widgetFile(filename)
            with open(file, "a") as fp:
                fp.write(customWidgetClass(_class,qt_class))

            self.project.settings['customWidgets'][filename].append({'class':_class,'qt_class':qt_class})
            self.project.saveSett()
            self.refresh()

        self.setMainPageIndex(2)
        self.ui.statusbar.showMessage(f'Custom widget "{_class}" created!')

    # Ui Methods
    def openUi(self,item):
        """Opens ui file in QDesigner"""
        self.ui.statusbar.showMessage('Opening in QDesigner...')
        qd = self.settings.value("usrInput/designerPath", None)
        if os.path.isdir(qd):
            os.chdir(qd)
            subprocess.Popen(['designer', item.file],shell=True)

    def addUiFiles(self,file):
        """Adds ui file to project"""
        file = self.openFiles('Ui Files (*.ui)')[0]\
         if not file else file
        [self.addUi(ui,True) for ui in file if len(file)]

    def createUi(self):
        ui_type = self.ui.newUiType.currentText()# ex: QMainWindow
        ui_name = self.ui.newUiInput.text()
        ui_name = ui_name if len(ui_name) else 'MainWindow'
        clean_name=Mt.cleanString(ui_name)+'.ui'

        if clean_name in self.project.settings['uiFiles']:
            self.ui.newUiInput.setText(clean_name.replace('.ui',''))
            self.extraMsg(f'Ui with same name: "{clean_name}" found! Please rename ui!')
            return

        ui_filename = 'mainwindow.ui' if ui_type == 'QMainWindow' else 'widget.ui'
        ui_file = os.path.join(MadQt.get_path('Templates'),'Uis',ui_filename)
        # copy template to tempDir
        temp_dir = os.path.join(tempDir(),clean_name)
        shutil.copyfile(ui_file,temp_dir)
        self.addUi(temp_dir,True)
        self.setMainPageIndex(2)

    def addUi(self,file=None,save=False):
        """
            Adds ui to the project ui list
            Saves ui to project settings if save=True
        """
        if isinstance(file,QListWidgetItem):
            file=file.file
            save=True
        elif not file.endswith('ui'):
            self.extraMsg('Not a valid ui file!')
            return

        # File was deleted from system?
        if not os.path.isfile(file):
            self.removeUi(file,True)
            return

        # Ui name already added?
        for item in self.ui.uiList.getItems():
            bname = os.path.basename(file)
            if item.name == bname:
                self.extraMsg(f'Ui with same name: "{bname}" found! Please rename ui!')
                return

        # Add ui file to dev folder if a dropped or added ui
        if save:
            file_in_dev = self.project.guiFile(os.path.basename(file))
            if not os.path.isfile(file_in_dev):
                t = os.path.join(tempDir(),os.path.basename(file))
                # if os.path.isfile(t):os.remove(t)
                if t != file:shutil.copy2(file,t)
                file = t

        # Add ui to the project ui list
        if not isTemp(file): self.ui.uiList.insertItem(0,UiItem(file))

        # Save ui to project settings
        if not save:return
        new_location = self.project.addUi(file)
        if isTemp(file): self.ui.uiList.insertItem(0,UiItem(new_location))

    def removeUi(self,file=None,save=False):
        """
            Removes item from the project page ui list
            Removes ui from project settings if save=True
        """
        if isinstance(file,QListWidgetItem):
            file=file.file
            save=True
        elif file is None:
            answer = self.questionBox("This will delete the .ui and compiled .py files from your system!\n Do you wish to proceed?")
            if answer != 'Yes':return
            [self.removeUi(ui) for ui in self.ui.uiList.selectedItems()]
        # Remove from uiList
        self.ui.uiList.removeItem(file)
        # Remove from project settings
        if not save:return
        self.project.removeUi(file)

    # Settings Methods
    def saveUserSettings(self):
        """save settings ui button"""
        dp = self.ui.QtDesignerPathInput.text()
        sp = self.ui.sublimePathInput.text()
        if os.path.isdir(dp):
            self.settings.setValue("usrInput/designerPath", dp)
        if os.path.isdir(sp):
            self.settings.setValue("usrInput/sublimePath", sp)
            self.ui.sublimePBtn.setStatusTip('Open sublime project')
            if self.project.valid:self.ui.sublimePBtn.setEnabled(1)

        self.setMainPageIndex(None)

    def defaultIcoSize(self):
        size = int(self.settings.value("usrInput/icoSize",str(64)))
        return [(size,size)]

    def addPluginPath(self):
        global MadQt_Designer_Paths
        options = QFileDialog.DontResolveSymlinks | QFileDialog.ShowDirsOnly
        directory = QFileDialog.getExistingDirectory(self,
                "MadQt Project Manager - Select plugin folder",
                MadQt.get_path('QtDesignerPlugins'), options)
        if directory:
            paths = self.settings.value("usrInput/pluginPaths",[])
            paths.append(directory)
            self.settings.setValue("usrInput/pluginPaths",paths)
            self.ui.designerPathsList.addItem(directory)
            MadQt_Designer_Paths=paths

    def removePluginPath(self):
        global MadQt_Designer_Paths
        paths = self.settings.value("usrInput/pluginPaths",[])
        removed = self.ui.designerPathsList.removeCurrentItem()
        for item in paths:
            if item == removed.text():
                paths.remove(item)
                break
        self.settings.setValue("usrInput/pluginPaths",paths)
        MadQt_Designer_Paths=paths

    # other
    def openSublimeProject(self):
        """opens sublime project if not open"""
        subDir = self.ui.sublimePathInput.text()
        if not os.path.isdir(subDir):return
        # change to sublime dir
        os.chdir(subDir)

        foundProject=False
        for file in os.listdir(self.project.folder):
            fn,ext = os.path.splitext(file)
            if 'sublime-project' in ext:
                foundProject = os.path.join(self.project.folder,file)
                break

        if not foundProject:
            msg = "A sublime-project file was not found! \n"
            msg += "MadQt will start a new sublime project for you,\n"
            msg += "but you will have to save it to your MadQt project folder. \n"
            msg += "\nCopy the path bellow and press Ok, when sublime opens,\n"
            msg += " go to: Project -> Save Project As... \n"
            msg += " navigate to the provided path, and save your file there!"
            usrIn = self.labelBox(msg,self.project.folder)
            if usrIn:
                # open new window
                nw = subprocess.run('subl -n',capture_output=True)
                # add folder
                addF = subprocess.run(['subl', '-b', '--command <Project:Add Folder>',
                    self.project.folder],shell=True,capture_output=True)
        else:
            subprocess.run(['subl', '--project', foundProject],shell=True)

    def hasSublime(self):
        """returns sublime text is available"""
        return os.path.isdir(self.ui.sublimePathInput.text())

    def runApp(self):
        if self.project.runApp() is not None:
            self.ui.statusbar.showMessage('Cannot run, no Ui added yet!')

def main():
    global window
    app = QApplication(sys.argv)
    window = App()
    window.show()
    sys.exit(app.exec())

if __name__ == '__main__':
    main()


# def sub_classes(_cls):
#     return [cls.__name__ for cls in _cls.__subclasses__()]

# def all_subclasses(cls):
#     return set(cls.__subclasses__()).union(
#         [s for c in cls.__subclasses__() for s in all_subclasses(c)])

# print(all_subclasses(QWidget))
# print(all_subclasses(QWidget))



""" MadQt - Tutorials and Tools for PyQt and PySide

    All the Code in this package can be used freely for personal and
    commercial projects under a MIT License but remember that because
    this is an extension of the PyQt framework you will need to abide
    to the QT licensing scheme when releasing.

    ## $MADQT_BEGIN_LICENSE
    ## MIT License
    ##
    ## Copyright (c) 2021 Fabio Goncalves
    ##
    ## Permission is hereby granted, free of charge, to any person obtaining a copy
    ## of this software and associated documentation files (the "Software"), to deal
    ## in the Software without restriction, including without limitation the rights
    ## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    ## copies of the Software, and to permit persons to whom the Software is
    ## furnished to do so, subject to the following conditions:
    ##
    ## The above copyright notice and this permission notice shall be included in all
    ## copies or substantial portions of the Software.
    ##
    ## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    ## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    ## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    ## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    ## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    ## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    ## SOFTWARE.
    ## $MADQT_END_LICENSE

    ## $QT_BEGIN_LICENSE:BSD$
    ## Commercial License Usage
    ## Licensees holding valid commercial Qt licenses may use this file in
    ## accordance with the commercial license agreement provided with the
    ## Software or, alternatively, in accordance with the terms contained in
    ## a written agreement between you and The Qt Company. For licensing terms
    ## and conditions see https://www.qt.io/terms-conditions. For further
    ## information use the contact form at https://www.qt.io/contact-us.
    ##
    ## BSD License Usage
    ## Alternatively, you may use this file under the terms of the BSD license
    ## as follows:
    ##
    ## "Redistribution and use in source and binary forms, with or without
    ## modification, are permitted provided that the following conditions are
    ## met:
    ##   * Redistributions of source code must retain the above copyright
    ##     notice, this list of conditions and the following disclaimer.
    ##   * Redistributions in binary form must reproduce the above copyright
    ##     notice, this list of conditions and the following disclaimer in
    ##     the documentation and/or other materials provided with the
    ##     distribution.
    ##   * Neither the name of The Qt Company Ltd nor the names of its
    ##     contributors may be used to endorse or promote products derived
    ##     from this software without specific prior written permission.
    ##
    ##
    ## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    ## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    ## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
    ## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
    ## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
    ## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    ## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    ## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    ## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    ## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
    ##
    ## $QT_END_LICENSE$
"""




############################################################################
    ##
    ## Copyright (C) 2021 The Qt Company Ltd.
    ## Contact: https://www.qt.io/licensing/
    ##
    ## This file is part of the Qt for Python examples of the Qt Toolkit.
    ##
    ## $QT_BEGIN_LICENSE:BSD$
    ## Commercial License Usage
    ## Licensees holding valid commercial Qt licenses may use this file in
    ## accordance with the commercial license agreement provided with the
    ## Software or, alternatively, in accordance with the terms contained in
    ## a written agreement between you and The Qt Company. For licensing terms
    ## and conditions see https://www.qt.io/terms-conditions. For further
    ## information use the contact form at https://www.qt.io/contact-us.
    ##
    ## BSD License Usage
    ## Alternatively, you may use this file under the terms of the BSD license
    ## as follows:
    ##
    ## "Redistribution and use in source and binary forms, with or without
    ## modification, are permitted provided that the following conditions are
    ## met:
    ##   * Redistributions of source code must retain the above copyright
    ##     notice, this list of conditions and the following disclaimer.
    ##   * Redistributions in binary form must reproduce the above copyright
    ##     notice, this list of conditions and the following disclaimer in
    ##     the documentation and/or other materials provided with the
    ##     distribution.
    ##   * Neither the name of The Qt Company Ltd nor the names of its
    ##     contributors may be used to endorse or promote products derived
    ##     from this software without specific prior written permission.
    ##
    ##
    ## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    ## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    ## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
    ## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
    ## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
    ## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    ## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    ## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    ## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    ## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
    ##
    ## $QT_END_LICENSE$
    ##
############################################################################
import os
CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))

from template import Template

from PySide6.QtGui import QIcon
from PySide6.QtDesigner import (QExtensionManager, QDesignerCustomWidgetInterface)

DOM_XML = """
<ui language='c++'>
    <widget class='Template' name='template'/>
</ui>
"""

class TemplatePlugin(QDesignerCustomWidgetInterface):
    def __init__(self):
        super().__init__()
        self._form_editor = None

    def createWidget(self, *args, **kwargs):
        t = Template(*args, **kwargs)
        return t

    def domXml(self):
        return DOM_XML

    def group(self):
        return 'Custom Widgets'

    def icon(self):
        imgLocation = os.path.join(CURRENT_DIR,'IconName')
        return QIcon(imgLocation)

    def includeFile(self):
        return 'template'

    def initialize(self, form_editor):
        self._form_editor = form_editor

    def isContainer(self):
        return False

    def isInitialized(self):
        return self._form_editor is not None

    def name(self):
        return 'Template'

    def toolTip(self):
        return 'tooltip'

    def whatsThis(self):
        return self.toolTip()



#############################################################################
    ##
    ## Copyright (C) 2021 The Qt Company Ltd.
    ## Contact: http://www.qt.io/licensing/
    ##
    ## This file is part of the Qt for Python examples of the Qt Toolkit.
    ##
    ## $QT_BEGIN_LICENSE:BSD$
    ## You may use this file under the terms of the BSD license as follows:
    ##
    ## "Redistribution and use in source and binary forms, with or without
    ## modification, are permitted provided that the following conditions are
    ## met:
    ##   * Redistributions of source code must retain the above copyright
    ##     notice, this list of conditions and the following disclaimer.
    ##   * Redistributions in binary form must reproduce the above copyright
    ##     notice, this list of conditions and the following disclaimer in
    ##     the documentation and/or other materials provided with the
    ##     distribution.
    ##   * Neither the name of The Qt Company Ltd nor the names of its
    ##     contributors may be used to endorse or promote products derived
    ##     from this software without specific prior written permission.
    ##
    ##
    ## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    ## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    ## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
    ## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
    ## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
    ## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    ## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    ## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    ## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    ## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
    ##
    ## $QT_END_LICENSE$
    ##
#############################################################################
from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *

class Template(QWidget):
    currentIndexChanged = Signal(int)
    pageNameChanged = Signal(str)
    def __init__(self, parent=None):
        super().__init__(parent)

        #PROPERTY_VALUES#
        self._pages = []
        self._currentIndex = -1

        # Creating main layout
        l=QVBoxLayout(self)
        l.setContentsMargins(0,0,0,0)
        l.setSpacing(0)

        # Creating prev/next button layout
        bl=QBoxLayout(QBoxLayout.LeftToRight)

        # Adding previous button
        pb=QToolButton(self)
        # Using __qt__passive_ prefix to make sure
        # the button works in QtDesigner
        pb.setObjectName('__qt__passive_prevButton')
        pb.setText('<')
        pb.clicked.connect(self.previousPage)
        bl.addWidget(pb)

        # Adding next button
        nb=QToolButton(self)
        nb.setObjectName('__qt__passive_nextButton')
        nb.setText('>')
        nb.clicked.connect(self.nextPage)
        bl.addWidget(nb)

        # Adding buttons layout to main layout
        l.addLayout(bl)
        l.setAlignment(bl,Qt.AlignRight)

    #PROPERTY_SETTER#

    #PROPERTY_GETTER#

    @Slot()# this will insure this method will show in QtDesigner slots
    def nextPage(self):
        """
            Sets next page, if current page is last page,
            it sets current page as page 0
        """
        ci = self.getCurrentIndex()
        index = 0 if ci == self.count()-1 else ci+1
        self.setCurrentIndex(index)

    @Slot()
    def previousPage(self):
        """
            Sets previous page, if current page is first page,
            it sets current page as last page
        """
        ci = self.getCurrentIndex()
        index = self.count()-1 if ci == 0 else ci-1
        self.setCurrentIndex(index)

    def displayPage(self, index):
        """displays requested page"""
        for i, p in enumerate(self._pages):
            if i == index:
                self.layout().addWidget(p)
                p.show()
            else:
                self.layout().removeWidget(p)
                p.hide()

    def pageExists(self, index):
        return index < self.count() and index != -1

    def canRemovePage(self, index):
        return self.pageExists(index) and self.count()

    @Slot(int, QWidget)
    def insertPage(self, index, page=None):
        """
            Inserts page into requested index

            This method is requested by the Container extension
            page is a QWidget added by QtDesigner and it's parent
            is self (this widget).
        """
        if not page:page = QWidget()

        # Adding given page to the layout
        self.layout().addWidget(page)

        # getting the object name, if None we set it
        objName = page.objectName()
        if objName == "":
            objName = "Page %d" % (self.count() + 1)
            page.setObjectName(objName)

        # Inserting page into our page list
        self._pages.insert(index, page)

        self.setCurrentIndex(index)
        self.setPageName(objName)

        return page

    @Slot(QWidget)
    def addPage(self, page=None):
        if not page: page = QWidget()
        return self.insertPage(self.count(), page)

    def removePage(self, index):
        if not self.canRemovePage(index): return
        newIndex = max(self.getCurrentIndex()-1,0)
        self._pages.pop(index)
        self.setCurrentIndex(newIndex)

    def widget(self, index):
        return self._pages[index] if self._pages else None

    def currentPage(self):
        ci=self.getCurrentIndex()
        if self.pageExists(ci):
            return self._pages[ci]

    def getCurrentIndex(self):
        return self._currentIndex

    @Slot(int)
    def setCurrentIndex(self, index):
        index = min(max(index,0),self.count()-1)
        if index != self.getCurrentIndex():
            self._currentIndex = index
            self.displayPage(index)
            self.currentIndexChanged.emit(index)

    def getPageName(self):
        cp = self.currentPage()
        if cp: return cp.objectName()

    @Slot(str)
    def setPageName(self, newName):
        cp = self.currentPage()
        if cp:
            cp.setObjectName(newName)
            self.pageNameChanged.emit(newName)

    def count(self):
        return len(self._pages)

    def minimumSizeHint(self):
        return QSize(140, 140)

    def sizeHint(self):
        return QSize(140, 140)

    currentIndex = Property(int, getCurrentIndex, setCurrentIndex)
    currentPageName = Property(str, getPageName, setPageName, stored=False)
    #PROPERTY_ASSIGN#

if __name__ == '__main__':
    import sys
    app = QApplication(sys.argv)
    window = Template()

    # Adding page 1
    page1=window.addPage()
    # Adding a button to page 1
    QPushButton('Page 1 Button',page1)

    # Adding page 2
    page2=window.addPage()
    # Adding a button to page 2
    QPushButton('Page 2 Button',page2)

    # Setting widget to page 1
    window.setCurrentIndex(0)

    window.show()
    sys.exit(app.exec())



# Ui loader/compiler
import subprocess
import xml.etree.ElementTree as xml
class PySide6Ui:
    """
        class to load .ui files to memory or
        convert them to .py files

        based on:
        http://stackoverflow.com/a/14195313/3781327

        usage:
        PySide6Ui('myUi.ui').toPy('myUi.py')
        PySide6Ui('myUi.ui').toPy()

        PySide6Ui('myUi.ui').load()
    """
    def __init__(self, ui_file):
        self.__ui_file = ui_file

    def __getUi(self):
        return subprocess.check_output(['pyside6-uic', self.__ui_file])

    def toPy(self, py_file=None):
        py_file = py_file or self.__ui_file.replace('.ui','.py')
        uipy = self.__getUi()
        try:
            # Write the file
            with open(py_file, 'w') as f:
                f.write(uipy.decode("utf-8"))
            return True
        except:
            return False

    def load(self):
        uipy = self.__getUi()
        parsed = xml.parse(self.__ui_file)
        widget_class = parsed.find('widget').get('class')
        form_class = parsed.find('class').text
        pyc = compile(uipy, '<string>', 'exec')
        frame = {}
        exec(pyc, frame)
        form_class = frame['Ui_%s'%form_class]# Ui_MainWindow or Ui_Form
        base_class = eval('%s'%widget_class)# QMainWindow or QWidget
        return form_class, base_class



"""
Widget: HtButton
Version: 0.0.1

Contributors: Fabio Goncalves
Email: fabiogoncalves@live.co.uk

Description: A QPushButton with a hovered state and text
"""
from PySide6.QtCore import Signal, Slot, QSize, Property
from PySide6.QtWidgets import QPushButton, QApplication

class HtButton(QPushButton):
    hoveredTextChanged = Signal(str)
    defaultTextChanged = Signal(str)
    hoverStateChanged = Signal(bool)
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._hovered = False
        self._defaultText = 'Hover Me'
        self._hoveredText = 'Hello'
        self.setText(self._defaultText)

    @Slot(bool)
    def setHovered(self, new_hovered):
        self._hovered = new_hovered

    @Slot(str)
    def setHoveredText(self, new_hoveredText):
        self._hoveredText = new_hoveredText
        self.hoveredTextChanged.emit(new_hoveredText)

    @Slot(str)
    def setDefaultText(self, new_defaultText):
        self._defaultText = new_defaultText
        self.defaultTextChanged.emit(new_defaultText)

    def getHovered(self):
        return self._hovered

    def getHoveredText(self):
        return self._hoveredText

    def getDefaultText(self):
        return self._defaultText

    def minimumSizeHint(self):
        return QSize(75, 24)

    def sizeHint(self):
        return QSize(75, 24)

    def enterEvent(self,event):
        self.setText(self._hoveredText)
        self._hovered = True
        self.hoverStateChanged.emit(True)

    def leaveEvent(self,event):
        self.setText(self._defaultText)
        self._hovered = False
        self.hoverStateChanged.emit(False)

    # hovered = Property(bool, getHovered, setHovered)
    defaultText = Property(str, getDefaultText, setDefaultText)
    hoveredText = Property(str, getHoveredText, setHoveredText)
    hovered = Property(bool, getHovered, setHovered)

if __name__ == '__main__':
    import sys
    app = QApplication(sys.argv)
    window = HtButton()
    window.show()
    sys.exit(app.exec())



#############################################################################
    ##
    ## Copyright (C) 2021 The Qt Company Ltd.
    ## Contact: http://www.qt.io/licensing/
    ##
    ## This file is part of the Qt for Python examples of the Qt Toolkit.
    ##
    ## $QT_BEGIN_LICENSE:BSD$
    ## You may use this file under the terms of the BSD license as follows:
    ##
    ## "Redistribution and use in source and binary forms, with or without
    ## modification, are permitted provided that the following conditions are
    ## met:
    ##   * Redistributions of source code must retain the above copyright
    ##     notice, this list of conditions and the following disclaimer.
    ##   * Redistributions in binary form must reproduce the above copyright
    ##     notice, this list of conditions and the following disclaimer in
    ##     the documentation and/or other materials provided with the
    ##     distribution.
    ##   * Neither the name of The Qt Company Ltd nor the names of its
    ##     contributors may be used to endorse or promote products derived
    ##     from this software without specific prior written permission.
    ##
    ##
    ## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    ## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    ## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    ## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
    ## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
    ## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
    ## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    ## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    ## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    ## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    ## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
    ##
    ## $QT_END_LICENSE$
    ##
#############################################################################

from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *

class Template(QWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        #PROPERTY_VALUES#

    #PROPERTY_SETTER#

    #PROPERTY_GETTER#

    def minimumSizeHint(self):
        return QSize(200, 200)

    def sizeHint(self):
        return QSize(200, 200)

    #PROPERTY_ASSIGN#

if __name__ == '__main__':
    import sys
    app = QApplication(sys.argv)
    window = Template()
    window.show()
    sys.exit(app.exec())
